#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# fitnesseSettings
# @Since: 23-OCT-2019
# @Author: Jac. Beekers
# @Version: 20191023.0 - JBE - Initial
##

import cicd.fitnesse.fitnesseConstants as constants
import supporting, os, logging
import supporting.generalSettings as generalsettings
from supporting.generalSettings import completePath

logger = logging.getLogger(__name__)

sourcefitnessedir = constants.DEFAULT_SOURCE_FITNESSEDIR
targetfitnessedir = constants.DEFAULT_TARGET_FITNESSEDIR

def getfitnesseenvvars():
    thisproc="getfitnesseenvvars"
    global fitnessedeploylist, sourcefitnessedir, targetfitnessedir
    supporting.log(logger, logging.DEBUG, thisproc, 'started')
    fitnessedeploylist = completePath(os.environ.get(constants.varFitNesseDeployList, constants.DEFAULT_FITNESSE_DEPLOYLIST), generalsettings.sourceDir)
    sourcefitnessedir = completePath(os.environ.get(constants.varSourceFitNesseDir, constants.DEFAULT_SOURCE_FITNESSEDIR), generalsettings.sourceDir)
    targetfitnessedir = completePath(os.environ.get(constants.varTargetFitNesseDir, constants.DEFAULT_TARGET_FITNESSEDIR), generalsettings.sourceDir)


def outfitnesseenvvars():
    thisproc = "outfitnesseenvvars"
    supporting.log(logger, logging.INFO,  thisproc, 'fitnessedeploylist is >' + fitnessedeploylist + "<.")
    supporting.log(logger, logging.INFO,  thisproc, 'sourcefitnessedir is >' + sourcefitnessedir +"<.")
    supporting.log(logger, logging.INFO,  thisproc, 'targetfitnessedir is >' + targetfitnessedir +"<.")





#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

#  MIT License
#
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#
#

import logging, datetime, supporting
from supporting import errorcodes
from cicd.informatica import infaSettings
from cicd.informatica import infaConstants
from supporting import generalSettings
from cicd.informatica import manageConnection
import sys, argparse

now = datetime.datetime.now()
result = errorcodes.OK


def parse_the_arguments(argv):
    """Parses the provided arguments and exits on an error.
    Use the option -h on the command line to get an overview of the required and optional arguments.
     """
    parser = argparse.ArgumentParser()
    parser.add_argument("-o", "--outputfile", required=False, action="store", dest="output_file",
                        help="File the connections should be written to. Default value is >" + infaConstants.DEFAULT_CONNECTIONSFILE + "<.")
    args = parser.parse_args()

    if args.output_file is None:
        args.output_file = infaConstants.DEFAULT_CONNECTIONSFILE

    return args


def main(argv):
    """List the connections as available in the Informatica domain.
    Usage: listConnections.py [-h] [-o OUTPUT_FILE]
    If no output file is provided, the default will be taken from infaConstants.DEFAULT_CONNECTIONSFILE
    """
    thisproc = "MAIN"
    mainProc = 'listConnections'

    resultlogger = supporting.configurelogger(mainProc)
    logger = logging.getLogger(mainProc)

    args = parse_the_arguments(argv)

    generalSettings.getenvvars()

    supporting.log(logger, logging.DEBUG, thisproc, 'Started')
    supporting.log(logger, logging.DEBUG, thisproc, 'logDir is >' + generalSettings.logDir + "<.")

    output_file = args.output_file

    infaSettings.getinfaenvvars()
    infaSettings.outinfaenvvars()

    connection = manageConnection.ManageConnection(Tool="ListConnections",
                                                   Domain=infaSettings.sourceDomain,
                                                   OnError=errorcodes.INFACMD_LIST_CONN_FAILED,
                                                   OutputFile=output_file
                                                   )
    result = manageConnection.ManageConnection.manage(connection)
    if result.rc == errorcodes.OK.rc:
        result = connection.parseConnectionListOutput(output_file)
        if result.rc == errorcodes.OK.rc:
            result = connection.writeConnectionList(output_file)
        else:
            supporting.log(logger, logging.DEBUG, thisproc,
                           'parseConnectionListOutput completed with return code >' + str(result.rc)
                           + '< and result code >' + result.code + "<.")
    else:
        with open(output_file, 'r') as f:
            for line in f:
                result.message += line.rstrip()

    supporting.log(logger, logging.DEBUG, thisproc, 'Completed with return code >' + str(result.rc)
                   + '< and result code >' + result.code + "<.")
    supporting.exitscript(resultlogger, result)


if __name__ == '__main__':
    main(sys.argv[1:])



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# General constants for build and deploys
# @Since: 22-MAR-2019
# @Author: Jac. Beekers
# @Version: 20190410.0 - JBE - Initial

##
# Environment variable values
##
# Generic
varLogDir = 'LOGDIR'
DEFAULT_LOGDIR = '.'
varResultDir = 'RESULTDIR'
DEFAULT_RESULTDIR = '.'
varArtifactDir = 'ARTIFACTDIR'
DEFAULT_ARTIFACTDIR = '.'
varConfigDir = 'CONFIGDIR'
DEFAULT_CONFIGDIR = '.'
varSourceDir = 'SOURCEDIR'
DEFAULT_SOURCEDIR = '.'
varDoNotRun = 'DO_NOT_RUN'
DEFAULT_DO_NOT_RUN = 'False'

##
# pipeline
varWorkspace = 'PIPELINE_WORKSPACE'
DEFAULT_WORKSPACE = '.'

##
# Release
varReleaseId = 'BUILD_BUILDNUMBER'

##
#
NOT_SET = 'NotSet'
YES = 'Yes'
NO = 'No'
TRUE = "True"
FALSE = "False"

##
# Environment defaults
# If not set on command line (if supported) and not as environment variable
DEFAULT_RELEASEID = '0.1'

# Nexus
varGroupId = "General"



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

#  MIT License
#
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#
#

import logging, datetime, supporting
from supporting import errorcodes
from cicd.informatica import infaSettings
from supporting import generalSettings
from cicd.informatica import manageSecurity
import sys, argparse

now = datetime.datetime.now()
result = errorcodes.OK


def parse_the_arguments(argv):
    """Parses the provided arguments and exits on an error.
    Use the option -h on the command line to get an overview of the required and optional arguments.
     """
    parser = argparse.ArgumentParser()
    parser.add_argument("-o", "--outputfile", required=True, action="store", dest="output_file",
                        help="File to export the users and groups to. Tip: Include the path.")
    parser.add_argument("-f", "--force", required=False, action="store", dest="force",
                        choices=["false", "true"],
                        help="If the target file exists it will be overwritten (true) or not (false). Default is >false<.")
    parser.add_argument("-r", "--retainpassword", required=False, action="store", dest="retain_password",
                        choices=["false", "true"],
                        help="Determines if the user passwords should be exported (true) or not (false). Default is >true<.")
    args = parser.parse_args()

    if args.force is None:
        args.force = "false"

    if args.retain_password is None:
        args.retain_password = "true"

    return args


def main(argv):
    """Exports users and groups.
    usage: exportUsersAndGroups.py [-h] -o OUTPUT_FILE [-f {false,true}]
                               [-r {false,true}]
    where:
    -f or --force: Overwrite output file if it exists
    -r or --retainpassword: If set to "false" user passwords are not exported. If "true" they will be.
    """
    thisproc = "MAIN"
    mainProc = 'exportUsersAndGroups'

    resultlogger = supporting.configurelogger(mainProc)
    logger = logging.getLogger(mainProc)

    args = parse_the_arguments(argv)

    generalSettings.getenvvars()

    supporting.log(logger, logging.DEBUG, thisproc, 'Started')
    supporting.log(logger, logging.DEBUG, thisproc, 'logDir is >' + generalSettings.logDir + "<.")

    export_file_name = args.output_file
    force = args.force
    retain_password = args.retain_password

    infaSettings.getinfaenvvars()
    infaSettings.outinfaenvvars()

    users_and_groups = manageSecurity.ManageSecurity(Tool="ExportUsersAndGroups",
                                                     Domain=infaSettings.sourceDomain,
                                                     ExportFile=export_file_name,
                                                     Force=force,
                                                     RetainPassword=retain_password,
                                                     OnError=errorcodes.INFACMD_EXPORT_USRGRP_FAILED
                                                     )

    result = manageSecurity.ManageSecurity.manage(users_and_groups)

    supporting.log(logger, logging.DEBUG, thisproc, 'Completed with return code >' + str(result.rc)
                   + '< and result code >' + result.code + "<.")
    supporting.exitscript(resultlogger, result)


if __name__ == '__main__':
    main(sys.argv[1:])



#  MIT License
#
#  Copyright (c) 2020 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

from supporting import log
import logging
from cicd.informatica import buildCommand
from cicd.informatica import executeInfacmd
from supporting import errorcodes
from supporting.mask_password import mask_password
from cicd.informatica import infaConstants as constants
from cicd.informatica import infaSettings
import supporting
from supporting import filehandling
import sys


logger = logging.getLogger(__name__)

"""
    List Application objects
"""


def list(logical_dis_name):
    thisproc = 'list'
    """Runs Informatica command line to list applications deployed to a DIS
    """
    tmp_file_name = filehandling.generate_tmp_filename()
    supporting.log(logger, logging.DEBUG, thisproc, 'Started')

    actual_dis_name = infaSettings.get_dis_name(logical_dis_name)
    infaSettings.getinfaenvvars()

    run_command = buildCommand.build(
        Tool='ListApplications',
        Domain=infaSettings.sourceDomain,
        ServiceName=actual_dis_name,
        OutputFile=tmp_file_name
    )

    masked_run_command = mask_password(run_command)

    log(logger, logging.DEBUG, __name__, "RunCommand is >" + masked_run_command + "<.")
    result = executeInfacmd.execute(run_command)

    app_list = filehandling.convert_content_to_array(tmp_file_name)
    filehandling.removefile(tmp_file_name)

    supporting.log(logger, logging.DEBUG, thisproc, 'Completed')
    return result, app_list


def main(args):
    thisproc = 'main'
    supporting.log(logger, logging.DEBUG, thisproc, 'args is >' + args + '<.')
    print(args)
    result, app_list = list(args)
    supporting.log(logger, logging.DEBUG, thisproc, "result is >" + result.message + "<.")
    if app_list is None:
        supporting.log(logger, logging.ERROR, thisproc, 'app_list could not be determined.')
    else:
        for a in app_list:
            print(a)


if __name__ == '__main__':
    main(sys.argv[1])



#  MIT License
# 
#  Copyright (c) 2019 Jac. Beekers
# 
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
# 
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
# 
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
# 

##
# Process deploy list for Scheduler artifacts
# @Since: 25-OCT-2019
# @Author: Jac. Beekers
# @Version: 20191025.0 - JBE - Initial

import supporting.errorcodes as err
import supporting, logging
import os
from cicd.scheduler import schedulerConstants as constants
from cicd.scheduler import schedulerSettings as settings
import supporting.deploylist
from pathlib import Path
from cicd.scheduler.schedulerArtifactChecks import checkSchedulerEntryType
from supporting import filehandling
import supporting.generalSettings as generalSettings

logger = logging.getLogger(__name__)
entrynr = 0
level = 0


def processList(deployFile):
    latestError = err.OK
    result, deployItems = supporting.deploylist.getWorkitemList(deployFile)
    if result.rc == err.OK.rc:
        filehandling.copy_file(deployFile, generalSettings.artifactDir)
        filehandling.create_directory(settings.targetschedulertypedir)
        filehandling.create_directory(settings.targetschedulerdir)
        for deployEntry in supporting.deploylist.deployItems:
            result = processEntry(deployEntry)
            if result.rc != 0:
                latestError = result
    else:
        # if no deploy list, then that is just fine.
        if result.rc == err.IGNORE.rc:
            latestError = err.OK
        else:
            latestError = result

    return latestError


def processEntry(deployEntry):
    thisproc = "processEntry"
    result = err.OK
    supporting.log(logger, logging.DEBUG, thisproc, "Current directory is >" + os.getcwd() + "<.")
    supporting.log(logger, logging.DEBUG, thisproc, "Started to work on deploy entry >" + deployEntry + "<.")

    type, directory, file = deployEntry.split(':', 3)
    supporting.log(logger, logging.DEBUG, thisproc,
                   'Type is >' + type + '<, directory is >' + directory + '< and file is >' + file + '<')

    result = checkSchedulerEntryType(type)
    if result.rc != 0:
        supporting.log(logger, logging.DEBUG, thisproc,
                       "checkSchedulerEntryType returned >" + result.message + "<. Entry ignored.")
        return result

    filePath = Path(file)
    if filePath.is_file():
        supporting.log(logger, logging.DEBUG, thisproc, 'Found file >' + file + "<.")
        sourcedir = ""
    else:
        sourcedir = settings.sourceschedulerdir + "/"
        supporting.log(logger, logging.DEBUG, thisproc, 'schedule file >' + file + '< not found. Trying >'
                       + sourcedir + file + '<...')
        filePath = Path(sourcedir + file)
        if filePath.is_file():
            supporting.log(logger, logging.DEBUG, thisproc, 'Found schedule file >' + sourcedir + file + "<.")
        else:
            supporting.log(logger, err.SCHEDULERFILE_NF.level, thisproc,
                           "schedule file checked >" + sourcedir + file + "<. " + err.SCHEDULERFILE_NF.message)
            result = err.SCHEDULERFILE_NF
            return result

    if type == constants.JOBTYPE:
        supporting.log(logger, logging.DEBUG, thisproc, 'copying Control-M job type file >' + sourcedir + file
                       + "< to >" + settings.targetschedulertypedir + "<.")
        filehandling.copy_file(sourcedir + file, settings.targetschedulertypedir)
    elif type == constants.JOBASCODE:
        supporting.log(logger, logging.DEBUG, thisproc, 'copying Control-M jobascode file >' + sourcedir + file
                       + "< to >" + settings.targetschedulerdir + "<.")
        filehandling.copy_file(sourcedir + file, settings.targetschedulerdir)
    elif type == constants.DAGS:
        supporting.log(logger, logging.DEBUG, thisproc, 'copying Airflow dag file >' + sourcedir + file
                       + "< to >" + settings.targetschedulerdir + "/" + directory + "<.")
        filehandling.create_directory(settings.targetschedulerdir + "/" + directory)
        filehandling.copy_file(sourcedir + file, settings.targetschedulerdir + "/" + directory + "/")
    elif type == constants.PLUGINS:
        supporting.log(logger, logging.DEBUG, thisproc, 'copying Airflow plugin file >' + sourcedir + file
                       + "< to >" + settings.targetschedulerdir + "<.")
        filehandling.copy_file(sourcedir + file, settings.targetschedulertypedir)
    else:
        supporting.log(logger, logging.WARN, thisproc, 'invalid type >' + type + '<. Entry ignored.')

    supporting.log(logger, logging.DEBUG, thisproc,
                   "Completed with rc >" + str(result.rc) + "< and code >" + result.code + "<.")

    return result


def determinebaseSourceDirectory(type):
    if type == constants.DAGS or type == constants.JOBASCODE:
        return settings.sourceschedulerdir
    if type == constants.PLUGINS or type == constants.JOBTYPE:
        return settings.sourceschedulertypedir

    return constants.NOT_SET


def determinebaseTargetDirectory(type):
    if type == constants.DAGS or type == constants.JOBASCODE:
        return settings.targetschedulerdir
    if type == constants.PLUGINS or type == constants.JOBTYPE:
        return settings.targetschedulertypedir

    return constants.NOT_SET


def determineSourceDirectory(directory, type):
    thisproc = "determineSourceDirectory"

    # type_path = directory + "/" + type
    type_path = directory
    directoryPath = Path(type_path)
    if directoryPath.is_dir():
        supporting.log(logger, logging.DEBUG, thisproc, 'Found directory >' + type_path + "<.")
        directory = type_path
    else:
        sourceDir = determinebaseSourceDirectory(type) + "/"
        supporting.log(logger, logging.DEBUG, thisproc, 'directory >' + type_path + '< not found. Trying >'
                       + sourceDir + type_path + '<...')
        type_path = sourceDir + type_path
        directoryPath = Path(type_path)
        if directoryPath.is_dir():
            supporting.log(logger, logging.DEBUG, thisproc, 'Found directory >' + type_path + "<.")
        else:
            supporting.log(logger, err.SQLFILE_NF.level, thisproc,
                           "directory checked >" + type_path + "<. " + err.DIRECTORY_NF.message)
            result = err.DIRECTORY_NF
            return constants.NOT_SET, result

    return type_path, err.OK



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#
import logging

import supporting.deploylist
from cicd import informatica
from cicd.informatica import infaSettings
from cicd.informatica import infaConstants
from supporting import generalSettings
from supporting import errorcodes

logger = logging.getLogger(__name__)
entrynr = 0


def processList(what, deployFile):
    this_function = "processList"
    latest_result = errorcodes.OK
    overall_result = errorcodes.OK

    supporting.log(logger, logging.DEBUG, this_function, "deployfile is >" + deployFile + "<.")
    result, deploy_items = supporting.deploylist.getWorkitemList(deployFile)
    if result.rc == 0:
        if what == infaConstants.DEPLOY_APP:
            for deployEntry in deploy_items:
                latest_result = process_deploy_app_entry(what, deployEntry)
                if latest_result.rc != errorcodes.OK.rc:
                    overall_result = latest_result
        elif what == infaConstants.CREATE_APP:
            for deployEntry in deploy_items:
                latest_result = process_create_app_entry(what, deployEntry)
                if latest_result.rc != errorcodes.OK.rc:
                    overall_result = latest_result
        else:
            supporting.log(logger, logging.ERROR, this_function, 'Invalid "what" value >' + what + '.')
            overall_result = errorcodes.INVALID_DEPLOY_TYPE

        return overall_result
    else:
        supporting.log(logger, logging.ERROR, this_function, "Could not get deploylist")
        return errorcodes.FILE_NF


def create_iar_file(app_path):
    result = informatica.create_iar_file(
        Domain=infaSettings.sourceDomain,
        Repository=infaSettings.sourceModelRepository,
        ApplicationPath=app_path,
        OutputDirectory=generalSettings.artifactDir + "/",
    )

    return result


def deploy_iar_file(app_name, dis_name):
    informatica_app_dir = infaSettings.target_informatica_app_dir

    result = informatica.deploy_iar_file(
        Domain=infaSettings.targetDomain,
        Application=app_name,
        ServiceName=dis_name,
        FileName=informatica_app_dir + "/" + app_name + '.iar'
    )

    return result


def redeploy_iar_file(app_name, dis_name):
    informatica_app_dir = infaSettings.target_informatica_app_dir

    result = informatica.redeploy_iar_file(
        Domain=infaSettings.targetDomain,
        Application=app_name,
        ServiceName=dis_name,
        FileName=informatica_app_dir + "/" + app_name + '.iar'
    )

    return result


def stop_app(app_name, dis_name):
    result = informatica.stop_app(
        Domain=infaSettings.targetDomain,
        Application=app_name,
        ServiceName=dis_name,
    )

    return result


def start_app(app_name, dis_name):
    result = informatica.start_app(
        Domain=infaSettings.targetDomain,
        Application=app_name,
        ServiceName=dis_name,
    )

    return result


def process_create_app_entry(what, deployEntry):
    global entrynr
    thisproc = "process_create_app_entry"
    result = errorcodes.OK

    entrynr += 1
    supporting.log(logger, logging.DEBUG, thisproc,
                   "Started to work on deploy entry# >" + str(entrynr) + "< being >" + deployEntry + "<.")
    parts = deployEntry.split(':')
    if not len(parts) == 2:
        supporting.log(logger, logging.DEBUG, thisproc,
                       "Expected 2 arguments, got >" + str(len(parts)) + "<.")
        return errorcodes.IGNORE

    app_path = parts[0]
    supporting.log(logger, logging.DEBUG, thisproc, 'app_path is >' + app_path + '<')
    result = create_iar_file(app_path)

    return result


def process_deploy_app_entry(what, deployEntry):
    global entrynr
    thisproc = "process_deploy_app_entry"
    result = errorcodes.OK

    entrynr += 1
    supporting.log(logger, logging.DEBUG, thisproc,
                   "Started to work on deploy entry# >" + str(entrynr) + "< being >" + deployEntry + "<.")
    parts = deployEntry.split(':')
    if not len(parts) == 2:
        supporting.log(logger, logging.DEBUG, thisproc,
                       "Insufficient entries found. Expected 2, got >" + str(len(parts)) + "<.")
        return errorcodes.IGNORE

    app_path = parts[0]
    app_name = app_path.rsplit('/', 1)[1]
    logical_dis_name = parts[1]

    # find the actual DIS name
    actual_dis_name = infaSettings.get_dis_name(logical_dis_name)

    supporting.log(logger, logging.DEBUG, thisproc, 'app_name is >' + app_name + '<, logical_dis_name is >'
                   + logical_dis_name + '<, actual_dis_name is >' + actual_dis_name + '<.')

    # TODO: Check if app exists, if so, run stop_app, then redeploy_iar_file. If not, run deploy_iar_file
    result = deploy_iar_file(app_name, actual_dis_name)
    if result.rc != errorcodes.OK.rc:
        # Check the message if the failure is due to the fact the app already exists
        if infaConstants.informatica_app_already_exists in result.message:
            supporting.log(logger, logging.DEBUG, thisproc, 'Application >' + app_name
                           + '< already exists, will update it. (Code contains TODO to optimize the process)')
            stop_app(app_name, actual_dis_name)
            result = redeploy_iar_file(app_name, actual_dis_name)
            # Also needed? start_app(app_name, actual_dis_name)

    return result



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# Errorcodes
# @Since: 22-MAR-2019
# @Author: Jac. Beekers
# @Version: 20190322.0 - JBE - Initial

import logging
import supporting.errorcode as err

# Returncode, ErrorCode, ErrorMessage, Resolution, Area, Severity

OK = err.Errorcode(0, '0', 'No errors encountered.', 'No action needed.', 'Result', logging.INFO)
IGNORE = err.Errorcode(-1, 'GEN-0001', 'Ignored', 'No action needed.', 'Result', logging.WARNING)
##
# General errors, mostly related to environment and/or privileges
LOGDIR_NOTSET = err.Errorcode(1, 'ENV-0001', 'LOGDIR not defined', 'Set the environment variable', 'Environment',
                              logging.FATAL)
LOGDIR_NF = err.Errorcode(1, 'ENV-0002', 'LOGDIR not found', 'Set the environment variable to an existing directory',
                          'Environment', logging.FATAL)
LOGDIR_NW = err.Errorcode(1, 'ENV-0003', 'Cannot write to LOGDIR',
                          'Set the environment variable to a writeable directory', 'Environment', logging.FATAL)
FILE_NF = err.Errorcode(1, 'ENV-0004', 'Cannot find file', 'Check the file\'s path and permissions', 'Environment',
                        logging.ERROR)
FILE_NW = err.Errorcode(1, 'ENV-0005', 'Cannot write to file', 'Check the file\'s path and permissions', 'Environment',
                        logging.ERROR)
##
# General build-deploy-run related errors. Return code is 2
DEPLOYLIST_NF = err.Errorcode(2, 'GENDEPLOY-0001', 'Deploylist not found', 'Check config directory and file name.',
                              'DatabaseArtifact', logging.FATAL)
COMMAND_FAILED = err.Errorcode(2, 'GENRUN-0001', 'Command failed', 'Check command output', 'executeCommand',
                               logging.ERROR)

# Database artifact errors. Return code will be 10
NO_DEPLOYLIST = err.Errorcode(10, 'DBDEPLOY-0001', 'No dbdeploylist defined', 'Set the environment variable',
                              'DatabaseArtifact', logging.FATAL)
SOURCESQLDIR_NOTSET = err.Errorcode(10, 'DBDEPLOY-0002', 'SourceSqlDir is not defined', 'Set the environment variable',
                                    'DatabaseArtifact', logging.ERROR)
TARGETSQLDIR_NOTSET = err.Errorcode(10, 'DBDEPLOY-0003', 'TargetSqlDir is not defined', 'Set the environment variable',
                                    'DatabaseArtifact', logging.ERROR)
SQLFILE_NF = err.Errorcode(10, 'DBDEPLOY-0004', 'SQL file not found', 'Check the deploy file content',
                           'DatabaseArtifact', logging.ERROR)

##
# Directory based errors
DIRECTORY_NF = err.Errorcode(10, 'DIRDEPLOY-0001', 'Directory not found', 'Check the deploy file content',
                             'DirectoryArtifact', logging.ERROR)

##
# Database deploy errors
SQLPLUS_ERROR = err.Errorcode(10, 'DBDEPLOY-0005', 'sqlplus return an error.', 'Check the log output', 'DatabaseDeploy',
                              logging.ERROR)

# Informatica artifact errors. Return code will be 20
NOT_IMPLEMENTED = err.Errorcode(20, 'INFACICD-0001',
                                'Result unknown. Function may not have been implemented completely',
                                'Ask your developer to implement the logic completely.', 'InformaticaArtifact',
                                logging.WARNING)
INFACMD_FAILED = err.Errorcode(20, 'INFACICD-0002', 'infacmd command failed.',
                               'Check the log and/or ask your administrator.', 'InformaticaArtifact', logging.ERROR)
INFACMD_LIST_CONN_FAILED = err.Errorcode(20, 'INFACICD-0003', 'infacmd failed to list connections.',
                                         'Check the error message.', 'ListConnections', logging.ERROR)
INFACMD_LIST_CONN_OPTIONS_FAILED = err.Errorcode(20, 'INFACICD-0004', 'infacmd failed to list connection options.',
                                                 'Check the error message.', 'ListConnectionOptions', logging.ERROR)
INFACMD_NOCONNECTIONNAME = err.Errorcode(20, 'INFACICD-0005', 'No connection name provided.',
                                         'Provide a connection name for which you want the options to be listed.',
                                         'ListConnectionOptions', logging.ERROR)
INFACMD_EXPORT_CONN_FAILED = err.Errorcode(20, 'INFACICD-0006', 'Export Connections failed.',
                                           'Check the error message.', 'ExportConnections', logging.ERROR)
INFACMD_IMPORT_CONN_FAILED = err.Errorcode(20, 'INFACICD-0007', 'Import Connections failed.',
                                           'Check the error message.', 'ImportConnections', logging.ERROR)
INVALID_DEPLOY_TYPE = err.Errorcode(20, 'INFACICD-0008', 'INTERNAL ERROR: Invalid deploy type.',
                                    'Internal error: Contact the development team.', 'InformaticaApp', logging.ERROR)

# Informatica run errors. Return code will be 30
INFACMD_NOPROFILE = err.Errorcode(30, 'INFARUN-0001', 'No profile name provided.',
                                  'Provide the complete path of the profile to be executed.', 'RunProfile',
                                  logging.ERROR)
INFACMD_PROFILE_FAILED = err.Errorcode(30, 'INFARUN-0002', 'infacmd run profile command failed.',
                                       'Check the log and/or ask your administrator.', 'RunProfile', logging.ERROR)
INFACMD_NOSCORECARD = err.Errorcode(30, 'INFARUN-0003', 'No scorecard name provided.',
                                    'Provide the complete path of the scorecard to be executed.', 'RunScorecard',
                                    logging.ERROR)
INFACMD_SCORECARD_FAILED = err.Errorcode(30, 'INFARUN-0004', 'infacmd run scorecard command failed.',
                                         'Check the log and/or ask your administrator.', 'RunScorecard', logging.ERROR)
INFACMD_NOMAPPING = err.Errorcode(30, 'INFARUN-0005', 'No mapping provided', 'Provide the mapping you want to run.',
                                  'RunMapping', logging.ERROR)
INFACMD_NOAPPFORMAPPING = err.Errorcode(30, 'INFARUN-0006', 'Application of the mapping was not provided',
                                        'Provide the application that contains the mapping you want to run.',
                                        'RunMapping', logging.ERROR)
INFACMD_MAPPING_FAILED = err.Errorcode(30, 'INFARUN-0007', 'infacmd run mapping command failed.',
                                       'Check the log and/or ask your administrator.', 'RunMapping', logging.ERROR)
INFACMD_NOPROJECT = err.Errorcode(30, 'INFARUN-0008', 'No project name provided.',
                                  'Provide a name for the project to be created.', 'CreateProject', logging.ERROR)
INFACMD_NOFOLDER = err.Errorcode(30, 'INFARUN-0009', 'No project and/or folder name provided.',
                                 'Provide the project and a name for the folder to be created.', 'CreateFolder',
                                 logging.ERROR)
INFACMD_CREATE_FOLDER_FAILED = err.Errorcode(30, 'INFARUN-0010', 'Folder could not be created.',
                                             'Check the error message.', 'CreateFolder', logging.ERROR)
INFACMD_CREATE_PROJECT_FAILED = err.Errorcode(30, 'INFARUN-0011', 'Project could not be created.',
                                              'Check the error message.', 'CreateProject', logging.ERROR)
INFACMD_DELETE_PROJECT_FAILED = err.Errorcode(30, 'INFARUN-0012', 'Project could not be removed.',
                                              'Check the error message.', 'DeleteProject', logging.ERROR)
INFACMD_DELETE_FOLDER_FAILED = err.Errorcode(30, 'INFARUN-0012', 'Folder could not be removed.',
                                             'Check the error message.', 'DeleteFolder', logging.ERROR)
INFACMD_NOWORKFLOW = err.Errorcode(30, 'INFARUN-0013', 'No application, workflow and/or wait provided',
                                   'You need to specify the application name, workflow and whether to wait (true) or not (false).',
                                   'RunWorkflow', logging.ERROR)
INFACMD_WORKFLOW_FAILED = err.Errorcode(30, 'INFARUN-0014', 'Workflow failed.', 'Check the error message and logs.',
                                        'RunWorkflow', logging.ERROR)

##
# Manage Security errors
INFACMD_NOUSERNAME = err.Errorcode(30, 'INFASEC-0001', 'No user name, password and/or full name provided.',
                                   'Provide a name, password, and full name for the user to be created.', 'CreateUser',
                                   logging.ERROR)
INFACMD_NOUSERNAME_DELETION = err.Errorcode(30, 'INFASEC-0002', 'No user name provided.',
                                            'Provide the username to be deleted.', 'DeleteUser', logging.ERROR)
INFACMD_NOEXPORTFILENAME = err.Errorcode(30, 'INFASEC-0003', 'No export file name provided.',
                                         'Provide the name for the export file.', 'ExportUsersAndGroups', logging.ERROR)
INFACMD_NOIMPORTFILENAME = err.Errorcode(30, 'INFASEC-0004', 'No import file name provided.',
                                         'Provide the name of the file to be imported.', 'ImportUsersAndGroups',
                                         logging.ERROR)
INFACMD_CREATE_USER_FAILED = err.Errorcode(30, 'INFASEC-0005', 'User creation failed.', 'Check the error message.',
                                           'CreateUser', logging.ERROR)
INFACMD_DELETE_USER_FAILED = err.Errorcode(30, 'INFASEC-0006', 'User deletion failed.', 'Check the error message.',
                                           'DeleteUser', logging.ERROR)
INFACMD_CREATE_GROUP_FAILED = err.Errorcode(30, 'INFASEC-0007', 'Group creation failed.', 'Check the error message.',
                                            'CreateGroup', logging.ERROR)
INFACMD_DELETE_GROUP_FAILED = err.Errorcode(30, 'INFASEC-0018', 'Group deletion failed.', 'Check the error message.',
                                            'DeleteGroup', logging.ERROR)
INFACMD_EXPORT_USRGRP_FAILED = err.Errorcode(30, 'INFASEC-0019', 'Users and groups export failed.',
                                             'Check the error message.', 'ExportUsersAndGroups', logging.ERROR)
INFACMD_IMPORT_USRGRP_FAILED = err.Errorcode(30, 'INFASEC-0020', 'Users and groups import failed.',
                                             'Check the error message.', 'ImportUsersAndGroups', logging.ERROR)
INFACMD_ADD_PERM_FAILED = err.Errorcode(30, 'INFASEC-0021', 'Permissions could not be added.',
                                        'Check the error message.', 'AddPermissions', logging.ERROR)
INFACMD_REMOVE_PERM_FAILED = err.Errorcode(30, 'INFASEC-0023', 'Permissions could not be remvoed.',
                                           'Check the error message.', 'RemovePermissions', logging.ERROR)
INFACMD_SET_CONN_FAILED = err.Errorcode(30, 'INFASEC-0024', 'Permissions could not be set.', 'Check the error message.',
                                        'SetPermissions', logging.ERROR)
INFACMD_NOGROUPNAME = err.Errorcode(30, 'INFASEC-0025', 'No group name provided.',
                                    'Provide a name for the group to be created.', 'CreateGroup', logging.ERROR)
INFACMD_NOGROUPNAME_DELETION = err.Errorcode(30, 'INFASEC-0026', 'No group name provided.',
                                             'Provide the name of the group to be deleted.', 'DeleteGroup',
                                             logging.ERROR)

##
# Scheduler artifact errors
INVALID_SCHEDULER_ENTRY_TYPE = err.Errorcode(40, 'SCHDEPLOY-0001', 'Invalid scheduler entry type.',
                                             'Provide a valid scheduler entry type, eg. dags, jobascode, plugin. Check schedulerConstants.py for more.',
                                             'SchedulerArtifact', logging.ERROR)
SCHEDULERFILE_NF = err.Errorcode(41, 'SCHDEPLOY-0002', 'Scheduler file not found.',
                                 'Provide a valid scheduler file. Check the scheduler deploy list.',
                                 'SchedulerArtifact', logging.ERROR)



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# dbSettings
# @Since: 22-MAR-2019
# @Author: Jac. Beekers
# @Version: 20190414.0 - JBE - Initial
##

from cicd.informatica import infaConstants as constants
import supporting, os, logging
import supporting.generalSettings as generalsettings
from supporting.generalSettings import completePath

logger = logging.getLogger(__name__)


##
# getInfaEnvironment
def getinfaenvvars():
    thisproc = "getinfaenvvars"
    global infadeploylist, infa_app_deploylist, overwriteExportFile, target_informatica_dir, target_informatica_app_dir
    global sourceExportRefData, sourceInfaHome, sourceInfacmdLocation, sourceInfacmdCommand, sourceInfacmd, \
        sourceDomainInfa, sourceDomain, sourceModelRepository, sourceUsername, sourcePassword, sourceSecurityDomain, \
        sourceDIS, exportControlFile
    global targetExportRefData, targetInfaHome, targetInfacmdLocation, targetInfacmdCommand, targetInfacmd, \
        targetDomainInfa, targetDomain, targetModelRepository, targetUsername, targetPassword, targetSecurityDomain, \
        targetDIS, importControlFile

    supporting.log(logger, logging.DEBUG, thisproc, 'started')
    infadeploylist = completePath(
        os.environ.get(constants.varDeveloperDeployList, constants.DEFAULT_DEVELOPER_DEPLOYLIST),
        generalsettings.sourceDir)
    infa_app_deploylist = completePath(
        os.environ.get(constants.var_developer_app_deploylist, constants.DEFAULT_DEVELOPER_APP_DEPLOYLIST),
        generalsettings.sourceDir)

    overwriteExportFile = os.environ.get(constants.varOverwriteExportFile, constants.DEFAULT_OVERWRITE_EXPORT_FILE)

    sourceExportRefData = os.environ.get(constants.varSourceExportRefData, constants.DEFAULT_EXPORT_REFDATA)
    sourceInfaHome = os.environ.get(constants.varSourceInfaHome, constants.DEFAULT_INFA_HOME)
    sourceInfacmdLocation = os.environ.get(constants.varSourceInfacmdLocation, sourceInfaHome + "/server/bin")
    sourceDomain = os.environ.get(constants.varSourceDomain, constants.DEFAULT_DOMAIN)
    sourceModelRepository = os.environ.get(constants.varSourceModelRepository, constants.DEFAULT_MODEL_REPOSITORY)
    sourceUsername = os.environ.get(constants.varSourceUsername, constants.DEFAULT_USERNAME)
    sourcePassword = os.environ.get(constants.varSourcePassword, constants.DEFAULT_PASSWORD)
    sourceSecurityDomain = os.environ.get(constants.varSourceSecurityDomain, constants.DEFAULT_SECURITYDOMAIN)
    sourceDomainInfa = os.environ.get(constants.varSourceDomainInfa, sourceInfaHome + "/domains.infa")
    if os.name == 'nt':
        sourceInfacmdCommand = 'infacmd.bat'
    else:
        sourceInfacmdCommand = 'infacmd.sh'
    sourceInfacmd = sourceInfacmdLocation + '/' + sourceInfacmdCommand
    sourceDIS = os.environ.get(constants.varSourceDIS, constants.DEFAULT_DATAINTEGRATION_SERVICE)
    exportControlFile = completePath(
        os.environ.get(constants.varExportControlFile, constants.DEFAULT_EXPORT_CONTROLFILE), generalsettings.sourceDir)

    targetExportRefData = os.environ.get(constants.varTargetImportRefData, constants.DEFAULT_IMPORT_REFDATA)
    targetInfaHome = os.environ.get(constants.varTargetInfaHome, constants.DEFAULT_INFA_HOME)
    targetInfacmdLocation = os.environ.get(constants.varTargetInfacmdLocation, targetInfaHome + "/server/bin")
    targetDomain = os.environ.get(constants.varTargetDomain, constants.DEFAULT_DOMAIN)
    targetModelRepository = os.environ.get(constants.varTargetModelRepository, constants.DEFAULT_MODEL_REPOSITORY)
    targetUsername = os.environ.get(constants.varTargetUsername, constants.DEFAULT_USERNAME)
    targetPassword = os.environ.get(constants.varTargetPassword, constants.DEFAULT_PASSWORD)
    targetSecurityDomain = os.environ.get(constants.varTargetSecurityDomain, constants.DEFAULT_SECURITYDOMAIN)
    targetDomainInfa = os.environ.get(constants.varTargetDomainInfa, sourceInfaHome + "/domains.infa")
    if os.name == 'nt':
        targetInfacmdCommand = 'infacmd.bat'
    else:
        targetInfacmdCommand = 'infacmd.sh'
    targetInfacmd = targetInfacmdLocation + '/' + targetInfacmdCommand
    targetDIS = os.environ.get(constants.varTargetDIS, constants.DEFAULT_DATAINTEGRATION_SERVICE)
    target_informatica_dir = os.environ.get(constants.varTargetInformaticaDir, constants.DEFAULT_TARGET_INFORMATICADIR)
    target_informatica_app_dir = os.environ.get(constants.varTargetInformaticaAppDir
                                                , constants.DEFAULT_TARGET_INFORMATICA_APP_DIR)

    importControlFile = completePath(
        os.environ.get(constants.varImportControlFile, constants.DEFAULT_IMPORT_CONTROLFILE), generalsettings.sourceDir)

    supporting.log(logger, logging.DEBUG, thisproc, 'completed')
    return


def outinfaenvvars():
    thisproc = "outinfaenvvars"
    supporting.log(logger, logging.DEBUG, thisproc, 'started')
    # no need to show source env vars when deploying
    # supporting.log(logger, logging.INFO, thisproc, constants.varSourceInfaHome + ' =>' + sourceInfaHome + "<.")
    ##
    # etc. etc.

    supporting.log(logger, logging.DEBUG, thisproc, 'completed')
    return


def getpwcenvvars():
    thisproc = "getpwcenvvars"
    global pwcdeploylist
    supporting.log(logger, logging.DEBUG, thisproc, 'started')

    pwcdeploylist = os.environ.get(constants.varDeveloperDeployList, constants.DEFAULT_DEVELOPER_DEPLOYLIST)

    supporting.log(logger, logging.DEBUG, thisproc, 'completed')
    return


def outpwcenvvars():
    thisproc = "outPwcEnvironment"
    supporting.log(logger, logging.DEBUG, thisproc, 'started')
    supporting.log(logger, logging.INFO, thisproc, constants.varPowerCenterDeployList + ' =>' + pwcdeploylist + "<.")
    ##
    # etc. etc.

    supporting.log(logger, logging.DEBUG, thisproc, 'completed')
    return


def getinfaenvvars_old():
    thisproc = "getinfaenvvars"
    global deploylist
    supporting.log(logger, logging.DEBUG, thisproc, 'started')

    deploylist = os.environ.get(constants.varDeveloperDeployList, constants.DEFAULT_DEVELOPER_DEPLOYLIST)

    supporting.log(logger, logging.DEBUG, thisproc, 'completed')
    return


def outinfaenvvars_old():
    thisproc = "outinfaenvvars"
    supporting.log(logger, logging.DEBUG, thisproc, 'started')
    supporting.log(logger, logging.DEBUG, thisproc, deploylist)

    supporting.log(logger, logging.DEBUG, thisproc, 'completed')
    return


def get_dis_name(logical_dis_name):
    thisproc = 'get_dis_name'
    supporting.log(logger, logging.DEBUG, thisproc, 'started')

    supporting.log(logger, logging.DEBUG, thisproc, 'logical_dis_name is >' + logical_dis_name +'<.')

    actual_dis_name = os.environ.get(logical_dis_name, constants.NOT_FOUND)
    supporting.log(logger, logging.DEBUG, thisproc, 'actual_dis_name is >' + actual_dis_name +'<.')

    supporting.log(logger, logging.DEBUG, thisproc, 'completed')
    return actual_dis_name



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

import logging, datetime, supporting
from supporting import errorcodes
from cicd.informatica import infaSettings, jobManagement
from supporting import generalSettings
import sys
import argparse

now = datetime.datetime.now()
result = errorcodes.OK


class ExecuteInformaticaWorkflow:
    """
        Runs an Informatica Workflow
    """

    def __init__(self, argv, log_on_console=True):
        self.arguments = argv
        self.mainProc = 'runWorkflow'
        self.resultlogger = supporting.configurelogger(self.mainProc, log_on_console)
        self.logger = supporting.logger

    def parse_the_arguments(self, arguments):
        """Parses the provided arguments and exits on an error.
        Use the option -h on the command line to get an overview of the required and optional arguments.
        """
        parser = argparse.ArgumentParser(prog='runWorkflow')

        parser.add_argument("-a", "--application", required=True, action="store", dest="application_name",
                            help="Application that contains the object to run.")
        parser.add_argument("-w", "--workflow", help="Workflow to run.", required=True, action="store",
                            dest="workflow_name")
        parser.add_argument("-c", "--completion", help="Wait for workflow completion", action="store", dest="wait"
                            , choices=["True", "False"], default="False")
        parser.add_argument("-f", "--osprofile", action="store", dest="os_profile",
                            help="Informatica OSProfile to use.")
        parser.add_argument("-x", "-extra", action="store", dest="as_is_options",
                            help="any options to add. Make sure to use double-quotes!")
        args = parser.parse_args(arguments)

        if args.as_is_options is None:
            args.as_is_options = ""

        return args

    def runit(self, arguments):
        """Runs a Workflow.
        usage: runWorkflow.py [-h] -a APPLICATION_NAME -w WORKFLOW_NAME
                          [-c {True,False}] [-x AS_IS_OPTIONS]
        with AsIsOptions, you can speficy e.g. a parameter set
            Example:
            runMapping myApp myMapping Source 3 "-ParameterSet myParameterSet -OperatingSystemProfile myOSProfile"
            It is important to supply the AsIsOptions as one single string
        """
        thisproc = "runit"

        args = self.parse_the_arguments(arguments)

        generalSettings.getenvvars()

        supporting.log(self.logger, logging.DEBUG, thisproc, 'Started')
        supporting.log(self.logger, logging.DEBUG, thisproc, 'logDir is >' + generalSettings.logDir + "<.")

        application_name = args.application_name
        workflow_name = args.workflow_name
        wait = args.wait
        as_is_options = args.as_is_options
        os_profile = args.os_profile

        infaSettings.getinfaenvvars()
        infaSettings.outinfaenvvars()
        #        supporting.logentireenv()

        workflow = jobManagement.JobExecution(Tool="RunWorkflow",
                                              # this will translate to StartWorkflow for the infacmd
                                              Domain=infaSettings.sourceDomain,
                                              ServiceName=infaSettings.sourceDIS,
                                              Application=application_name,
                                              Workflow=workflow_name,
                                              Wait=wait,
                                              OnError=errorcodes.INFACMD_WORKFLOW_FAILED,
                                              OperatingSystemProfile=os_profile,
                                              AsIsOptions=as_is_options
                                              )
        result = jobManagement.JobExecution.manage(workflow)

        supporting.log(self.logger, logging.DEBUG, thisproc, 'Completed with return code >' + str(result.rc)
                       + '< and result code >' + result.code + "<.")
        return result


if __name__ == '__main__':
    infa = ExecuteInformaticaWorkflow(sys.argv[1:], log_on_console=True)
    result = infa.runit(infa.arguments)
    supporting.exitscript(infa.resultlogger, result)



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# Process deploy list for FitNesse artifacts
# @Since: 23-OCT-2019
# @Author: Jac. Beekers
# @Version: 20191023.0 - JBE - Initial

import supporting.errorcodes as err
import supporting, logging
import os
import cicd.fitnesse.fitnesseSettings as settings
import supporting.deploylist
from pathlib import Path
from supporting.generatezip import generate_zip
from supporting.generatezip import addto_zip
from supporting.filehandling import copy_file
import supporting.generalSettings as generalSettings

logger = logging.getLogger(__name__)
entrynr = 0
level = 0
previous_schema = 'AUQW&^D*AD&FS'


def processList(deployFile):
    latestError = err.OK
    result, deployItems = supporting.deploylist.getWorkitemList(deployFile)
    if result.rc == err.OK.rc:
        copy_file(deployFile, generalSettings.artifactDir)
        for deployEntry in supporting.deploylist.deployItems:
            result = processEntry(deployEntry)
            if result.rc != 0:
                latestError = result
    else:
        # if no deploy list, then that is just fine.
        if result.rc == err.IGNORE.rc:
            latestError = err.OK
        else:
            latestError = result
    return latestError


def processEntry(deployEntry):
    thisproc = "processEntry"
    result = err.OK
    supporting.log(logger, logging.DEBUG, thisproc, "Current directory is >" + os.getcwd() + "<.")
    supporting.log(logger, logging.DEBUG, thisproc, "Started to work on deploy entry >" + deployEntry + "<.")

    directory, suppress_zip = deployEntry.split(':', 2)
    supporting.log(logger, logging.DEBUG, thisproc,
                   'Directory is >' + directory + '< and suppress_zip is >' + suppress_zip + '<')
    zipfilename = settings.targetfitnessedir + "/" + directory.replace('/','_') + ".zip"
    supporting.log(logger, logging.DEBUG, thisproc, 'zipfilename is >' + zipfilename + "<.")

    directoryPath = Path(directory)
    if directoryPath.is_dir():
        supporting.log(logger, logging.DEBUG, thisproc, 'Found directory >' + directory + "<.")
        sourcefitnessedir = ""
    else:
        sourcefitnessedir = settings.sourcefitnessedir + "/"
        supporting.log(logger, logging.DEBUG, thisproc, 'directory >' + directory + '< not found. Trying >'
                       + sourcefitnessedir + directory + '<...')
        directory = sourcefitnessedir + directory
        directoryPath = Path(directory)
        if directoryPath.is_dir():
            supporting.log(logger, logging.DEBUG, thisproc, 'Found directory >' + directory + "<.")
        else:
            supporting.log(logger, err.SQLFILE_NF.level, thisproc,
                           "directory checked >" + directory + "<. " + err.DIRECTORY_NF.message)
            result = err.DIRECTORY_NF
            return result

    if suppress_zip == 'Y':
        supporting.log(logger, logging.DEBUG, thisproc, "zip files will be ignored.")
        result = generate_zip(sourcefitnessedir, directory, zipfilename, '*', 'zip')
        result = addto_zip(sourcefitnessedir, directory + '.wiki', zipfilename, '*', 'zip')
    else:
        supporting.log(logger, logging.DEBUG, thisproc, "zip files will be included.")
        result = generate_zip(sourcefitnessedir, directory, zipfilename)
        result = addto_zip(sourcefitnessedir, directory + '.wiki', zipfilename)

    supporting.log(logger, logging.DEBUG, thisproc,
                   "Completed with rc >" + str(result.rc) + "< and code >" + result.code + "<.")
    return result



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

from supporting import log
import logging
from cicd.informatica import buildCommand
from cicd.informatica import executeInfacmd
from supporting import errorcodes
from supporting.mask_password import mask_password


class ManageSecurity:
    """Implements wrapper around security related commands, like create a user, a group and more."""

    def __init__(self, **keyword_arguments):
        self.logger = logging.getLogger(__name__)
        self.keyword_arguments = keyword_arguments

    def manage(self):
        """Runs Informatica command line to create, delete, update security related objects,
        like users and groups.
        """
        run_command = buildCommand.build(**self.keyword_arguments)

        masked_run_command = mask_password(run_command)

        log(self.logger, logging.DEBUG, __name__, "RunCommand is >" + masked_run_command + "<.")
        result = executeInfacmd.execute(run_command)

        if (result.rc != errorcodes.OK.rc):
            oldResult = result.message
            result = self.keyword_arguments["OnError"]
            result.message = oldResult

        return (result)



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# dbSettings
# @Since: 22-MAR-2019
# @Author: Jac. Beekers
# @Version: 20190410.0 - JBE - Initial
# @Version: 20190817.0 - JBE - Added user/password functionality
##

import cicd.database.dbConstants as constants
import supporting, os, logging
import supporting.generalSettings as generalsettings
from supporting.generalSettings import completePath

logger = logging.getLogger(__name__)

sourcesqldir = constants.DEFAULT_SOURCE_SQLDIR
targetsqldir = constants.DEFAULT_TARGET_SQLDIR
databaseType = 'UNKNOWN'
sqlprefix = constants.DEFAULT_SQL_PREFIX
if os.name == 'nt':
    sqlplus_command = 'sqlplus.exe'
else:
    sqlplus_command = 'sqlplus'

def getdbenvvars():
    thisproc="getdbenvvars"
    global dbdeploylist, sourcesqldir, targetsqldir, sqlprefix
    supporting.log(logger, logging.DEBUG, thisproc, 'started')
    dbdeploylist = completePath(os.environ.get(constants.varOracleDeployList, constants.DEFAULT_ORACLE_DEPLOYLIST), generalsettings.sourceDir)
    sourcesqldir = completePath(os.environ.get(constants.varSourceSqlDir, constants.DEFAULT_SOURCE_SQLDIR), generalsettings.sourceDir)
    targetsqldir = completePath(os.environ.get(constants.varTargetSqlDir, constants.DEFAULT_TARGET_SQLDIR), generalsettings.sourceDir)
    # prefix for ordered sql files
    sqlprefix = os.environ.get(constants.varSqlPrefix, constants.DEFAULT_SQL_PREFIX)


def getschemaenvvars(schema):
    thisproc="getschemaenvvars"
    global database_user, database_schema, database_user_password, database_tns_name
    # Database user etc.
    database_user = os.environ.get(schema + '_' + constants.varOracleDatabaseUser, constants.NOT_SET)
    database_user_password = os.environ.get(schema + '_' + constants.varDatabaseUserPassword, None)
    database_schema = os.environ.get(schema + '_' + constants.varOracleSchemaName, constants.NOT_SET)
    database_tns_name = os.environ.get(schema + '_' + constants.varOracleTNSName, constants.NOT_SET)

    supporting.log(logger, logging.DEBUG, thisproc, 'completed')
    return database_tns_name, database_schema, database_user, database_user_password


def outdbenvvars():
    thisproc = "outdbenvvars"
    supporting.log(logger, logging.INFO,  thisproc, 'dbdeploylist is >' + dbdeploylist + "<.")
    supporting.log(logger, logging.INFO,  thisproc, 'sourcesqldir is >' + sourcesqldir +"<.")
    supporting.log(logger, logging.INFO,  thisproc, 'targetsqldir is >' + targetsqldir +"<.")


def outschemaenvvars():
    thisproc = "outdbenvvars"
    supporting.log(logger, logging.DEBUG, thisproc, 'database_tns_name is >' + database_tns_name +"<.")
    supporting.log(logger, logging.DEBUG, thisproc, 'database_schema is >' + database_schema +"<.")
    supporting.log(logger, logging.DEBUG, thisproc, 'database user is >' + database_user +"<.")
    if database_user_password is None:
        supporting.log(logger, logging.WARNING, thisproc, 'database_user_password is empty')
    else:
        supporting.log(logger, logging.DEBUG, thisproc, 'database_user_password has been determined.')




#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

#
# pip install pycryptodome
import base64
from Crypto.Hash import SHA256
from Crypto.Random import get_random_bytes
from Crypto.PublicKey import RSA
from Crypto.Cipher import AES, PKCS1_OAEP
from Crypto import Random
import os
import uuid
from cryptography.fernet import Fernet

def decrypt(key, source, decode=True):
    if decode:
        source = base64.b64decode(source.encode("utf-8"))
    key = SHA256.new(key).digest()  # use SHA-256 over our key to get a proper-sized AES key
    iv = source[:AES.block_size]  # extract the iv from the beginning
    decryptor = AES.new(key, AES.MODE_CBC, iv)
    data = decryptor.decrypt(source[AES.block_size:])  # decrypt
    padding = data[-1]  # pick the padding value from the end; Python 2.x: ord(data[-1])
    if data[-padding:] != bytes([padding]) * padding:  # Python 2.x: chr(padding) * padding
        raise ValueError("Invalid padding...")
    return data[:-padding]  # remove the padding


class Encryption():
    private_filename = "tmp_private"
    public_filename = "tmp_public"
    encrypted_session_key = ''
    nonce = ''
    tag = ''

    def encrypt_with_certificates(self, data, key_instance, encrypted_file):
        key = RSA.generate(2048)
        private_key = key.export_key()
        current_private_filename = Encryption.private_filename + str(key_instance) + ".tmp"
        file_out = open(current_private_filename, "wb")
        file_out.write(private_key)
        file_out.close()

        public_key = key.publickey().export_key()
        
        current_public_filename = Encryption.public_filename + str(key_instance) + ".tmp"
        file_out = open(current_public_filename, "wb")
        file_out.write(public_key)
        file_out.close()

        recipient_key = RSA.import_key(open(current_public_filename).read())
        session_key = get_random_bytes(16)

        # Encrypt the session key with the public RSA key
        cipher_rsa = PKCS1_OAEP.new(recipient_key)
        self.encrypted_session_key = cipher_rsa.encrypt(session_key)

        # Encrypt the data with the AES session key
        cipher_aes = AES.new(session_key, AES.MODE_EAX)
        self.nonce = cipher_aes.nonce
        encrypted_data, self.tag = cipher_aes.encrypt_and_digest(data.encode('utf-8'))

        file_out = open(encrypted_file, "wb")
        [ file_out.write(x) for x in (self.encrypted_session_key, self.nonce, self.tag, encrypted_data) ]

        file_out.close()
        return encrypted_data

    def decrypt_with_certificates(self, key_instance, encrypted_file):
        
        current_private_filename = Encryption.private_filename + str(key_instance) + ".tmp"
        private_key = RSA.import_key(open(current_private_filename).read())
        file_in = open(encrypted_file, "rb")
        encrypted_session_key, nonce, tag, encrypted_data = \
           [ file_in.read(x) for x in (private_key.size_in_bytes(), 16, 16, -1) ]

        # Decrypt the session key with the private RSA key
        cipher_rsa = PKCS1_OAEP.new(private_key)
        session_key = cipher_rsa.decrypt(encrypted_session_key)

        # Decrypt the data with the AES session key
        cipher_aes = AES.new(session_key, AES.MODE_EAX, nonce)
        data = cipher_aes.decrypt_and_verify(encrypted_data, tag)
        file_in.close()
        return data.decode('utf-8')

    def cleanup(self, key_instance):
        os.remove(Encryption.private_filename + str(key_instance) + ".tmp")
        os.remove(Encryption.public_filename + str(key_instance) + ".tmp")

    def get_key(self) -> bytes:
        key = Fernet.generate_key()  # store in a secure location
        return key.decode()

    def encrypt(self, message, key: bytes) -> bytes:
        return Fernet(key).encrypt(message.encode('utf-8'))

    def decrypt(self, token, key: bytes) -> bytes:
        return Fernet(key).decrypt(token).decode('utf-8')


def verify():
    encryption = Encryption()
    data = 'Hello encrypted world!'
    encrypted_data_filename = "verify.enc.tmp"
    key_instance = 1
    encrypted = encryption.encrypt_with_certificates(data, key_instance, encrypted_data_filename)
    decrypted = encryption.decrypt_with_certificates(key_instance, encrypted_data_filename)
    print(decrypted)
    os.remove(encrypted_data_filename)
    encryption.cleanup(key_instance)


#main()



#  MIT License
# 
#  Copyright (c) 2019 Jac. Beekers
# 
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
# 
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
# 
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
# 

##
# Create Scheduler Artifact
# @Since: 25-OCT-2019
# @Author: Jac. Beekers

import logging, datetime, supporting
import supporting.errorcodes as err
from cicd.scheduler import schedulerArtifactChecks as schedulerChecks
from cicd.scheduler import schedulerSettings as settings
from cicd.scheduler import artifact as artifact
#from cicd.scheduler import schedulerConstants as constants
import supporting.generalSettings as generalSettings
import sys, argparse

now = datetime.datetime.now()
result = err.OK


class CreateSchedulerArtifact:
    """
        Creates an artifact with Schedules from file system
    """

    def __init__(self, argv, log_on_console=True):
        self.arguments = argv
        self.mainProc = 'createSchedulerArtifact'
        self.resultlogger = supporting.configurelogger(self.mainProc, log_on_console)
        self.logger = supporting.logger

    def parse_the_arguments(self, arguments):
        """Parses the provided arguments and exits on an error.
        Use the option -h on the command line to get an overview of the required and optional arguments.
        """
        parser = argparse.ArgumentParser(prog='createSchedulerArtifact')
        args = parser.parse_args(arguments)

        return args

    def runit(self, arguments):
        """Creates a scheduler artifact.
        """
        thisproc = "runit"

        args = self.parse_the_arguments(arguments)

        generalSettings.getenvvars()

        supporting.log(self.logger, logging.DEBUG, thisproc, 'Started')
        supporting.log(self.logger, logging.DEBUG, thisproc, 'logDir is >' + generalSettings.logDir + "<.")

        # Check requirements for artifact generation
        generalSettings.getenvvars()
        settings.getschedulerenvvars()
        settings.outschedulerenvvars()

        result = schedulerChecks.schedulerartifactchecks()
        if result.rc == err.IGNORE.rc:
            # deploylist is not mandatory since 2020-02-09
            supporting.log(logging, result.level, thisproc, 'Artifact ignored.')
            result = err.OK
        else:
            if result.rc != 0:
                supporting.log(self.logger, logging.ERROR, thisproc,
                               'Scheduler Artifact Checks failed with >' + result.message + "<.")
                supporting.exitscript(self.resultlogger, result)
            else:
                result = artifact.processList(settings.schedulerdeploylist)

        supporting.log(self.logger, logging.DEBUG, thisproc, 'Completed with return code >' + str(result.rc)
                       + '< and result code >' + result.code + "<.")
        #    supporting.writeresult(resultlogger, result)
        return result


if __name__ == '__main__':
    create_artifact = CreateSchedulerArtifact(sys.argv[1:], log_on_console=True)
    result = create_artifact.runit(create_artifact.arguments)
    supporting.exitscript(create_artifact.resultlogger, result)



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# FitNesse Artifact Checks
# @Since: 23-OCT-2019
# @Author: Jac. Beekers
# @Version: 20191023.0 - JBE - Initial

import supporting.errorcodes as err
import supporting, logging
import cicd.fitnesse.fitnesseSettings as settings
from pathlib import Path

logger = logging.getLogger(__name__)


def fitnesseartifactchecks():
    thisproc = "fitnesseartifactchecks"
    supporting.log(logger, logging.DEBUG, thisproc, 'started')
    result = err.OK

    if not settings.fitnessedeploylist:
        supporting.log(logger, err.IGNORE.level, thisproc, err.NO_DEPLOYLIST.message)
        supporting.log(logger, err.IGNORE.level, thisproc, "Assuming FitNesse is NOT part of the solution.")
        result = err.IGNORE
    else:
        deploylistFile = Path(settings.fitnessedeploylist)
        if not deploylistFile.is_file():
            supporting.log(logger, err.IGNORE.level, thisproc,
                           "fitnessedeploylist is >" + settings.fitnessedeploylist + "<. "
                           + err.DEPLOYLIST_NF.message + " - FitNesse artifact IGNORED.")
            result = err.IGNORE

    supporting.log(logger, logging.DEBUG, thisproc, 'completed with >' + str(result.rc) + "<.")
    return result



#  MIT License
#
#  Copyright (c) 2019 ABN AMRO Bank N.V.
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#
from supporting import generalConstants
from supporting import generalSettings
from cicd.informatica import infaConstants
from cicd.informatica import infaSettings
from supporting import log
import logging

logger = logging.getLogger(__name__)
entrynr = 0


def build(**KeyWordArguments):
    """Build an IDQ command, return it as string
    Process the input aruguments to compose the IDQ command
    This is done by first creating a list of strings, that are then joined to form the actual
    command
    The syntax used is as follows:
    $InfaPath + $InfaProgram + $InfaCommand + $InfaArguments
    """

    procName = "build"
    # Create the list that will hold the parts of the command, in order
    InfaArguments = []

    # Process each input argument.
    # The Tool arguments are processed separately, because those have to go first
    # For the other arguments, the order does not matter, so they can be processed together
    for key, value in KeyWordArguments.items():
        log(logger, logging.DEBUG, procName, "key =>" + key + "<.")
        if isinstance(value, str):
            if key.lower().__contains__("password"):
                log(logger, logging.DEBUG, procName, "value =>" + "***" + "<.")
            else:
                log(logger, logging.DEBUG, procName, "value =>" + value + "<.")
        # If the argument is "Tool" , assign the value to the variable Tool, and lookup the Program and
        # Command in AvailableTools, assign those to InfaProgram, InfaCommand
        if key == "Tool":
            Tool = KeyWordArguments["Tool"]
            (InfaProgram, InfaCommand) = infaConstants.AvailableTools[value]
        elif key == "Project":
            projectName = value
            InfaArguments.append(infaConstants.AvailableArguments[key] + " " + '"' + value + '"')
        elif key == "ExportRefData":
            if value == generalConstants.YES:
                InfaArguments.append(
                    "-oo " + '"' + "rtm:disName=" + infaSettings.sourceDIS + ",codePage=UTF-8,refDataFile=" +
                    generalSettings.artifactDir + "/" + projectName + ".zip" + '"')
        # If the argument is anything else, look it up in AvailableArguments and add the found
        # value to InfaArguments
        elif key in infaConstants.AvailableArguments:
            InfaArguments.append(infaConstants.AvailableArguments[key] + " " + '"' + value + '"')
        elif value is not None and value != "NONE":
            if key == "AsIsOptions":
                new_value = value.lstrip('"').rstrip('"')
                log(logger, logging.DEBUG, procName, "stripped value =>" + new_value + "<.")
                InfaArguments.append(" " + new_value + " ")
            elif key == "OutputFile":
                InfaArguments.append(" >" + value + " ")
            elif key != "OnError":
                InfaArguments.append("-" + key + " " + '"' + value + '"')

    # Put all parts of the command in the same list, in correct order, and join them into one
    # string
    IDQCommandParts = [infaSettings.sourceInfacmd, InfaProgram, InfaCommand] + InfaArguments
    IDQCommand = " ".join(IDQCommandParts)

    return (IDQCommand)



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

#  MIT License
#
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#
#

import logging, datetime, supporting
from supporting import errorcodes
from cicd.informatica import infaSettings
from supporting import generalSettings
from cicd.informatica import manageSecurity
import sys, argparse

now = datetime.datetime.now()
result = errorcodes.OK


def parse_the_arguments(argv):
    """Parses the provided arguments and exits on an error.
    Use the option -h on the command line to get an overview of the required and optional arguments.
     """

    parser = argparse.ArgumentParser()
    parser.add_argument("-g", "--group", required=True, action="store", dest="group_name",
                        help="Name of the user group that needs to be created.")
    parser.add_argument("-d", "--description", required=False, action="store", dest="group_description",
                        help="The group description.")
    args = parser.parse_args()

    if args.group_description is None:
        args.group_description = "Created by createGroup.py on " + now.strftime('%Y-%m-%d %H:%M:%S.%f')

    return args


def main(argv):
    thisproc = "MAIN"
    mainProc = 'createGroup'

    resultlogger = supporting.configurelogger(mainProc)
    logger = logging.getLogger(mainProc)

    args = parse_the_arguments(argv)

    generalSettings.getenvvars()

    supporting.log(logger, logging.DEBUG, thisproc, 'Started')
    supporting.log(logger, logging.DEBUG, thisproc, 'logDir is >' + generalSettings.logDir + "<.")

    group_name = args.group_name
    group_description = args.group_description

    supporting.log(logger, logging.DEBUG, thisproc, 'Group name is >' + group_name + '<.')
    supporting.log(logger, logging.DEBUG, thisproc, 'Group description is >' + group_description + '<.')

    infaSettings.getinfaenvvars()
    infaSettings.outinfaenvvars()

    group = manageSecurity.ManageSecurity(Tool="CreateGroup",
                                          Domain=infaSettings.sourceDomain,
                                          GroupName=group_name,
                                          GroupDescription=group_description,
                                          OnError=errorcodes.INFACMD_CREATE_GROUP_FAILED
                                          )
    result = manageSecurity.ManageSecurity.manage(group)

    supporting.log(logger, logging.DEBUG, thisproc, 'Completed with return code >' + str(result.rc)
                   + '< and result code >' + result.code + "<.")
    supporting.exitscript(resultlogger, result)


if __name__ == '__main__':
    main(sys.argv[1:])



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

import logging, supporting
import supporting.errorcodes as err

logger = logging.getLogger(__name__)
deployItems = []

def getWorkitemList(deployList):
    thisproc = "processList"
    latestError = err.OK
    global entrynr
    entrynr = 0
    global level
    supporting.log(logger, logging.DEBUG, thisproc, "Started to work on deploy list >" + deployList + "<.")

    try:
        with open(deployList) as theList:
            for line in theList:
                entrynr += 1
                if line.startswith("#"):
                    supporting.log(logger, logging.DEBUG, thisproc, "Ignoring comment line >" + str(entrynr) + "<.")
                else:
                    line = line.rstrip('\n')
                    if line:
                        deployItems.append(line)
                        supporting.log(logger, logging.DEBUG, thisproc, "line >" + str(entrynr) +"< added to worklist.")
                    else:
                        supporting.log(logger, logging.DEBUG, thisproc, "Ignoring empty line >" + str(entrynr) +"<.")
    except IOError:
        supporting.log(logger, logging.ERROR, thisproc, "File not found")
        latestError = err.FILE_NF

    supporting.log(logger, logging.DEBUG, thisproc,
                   "Completed with rc >" + str(latestError.rc) + "< and code >" + latestError.code + "<.")
    return latestError, deployItems



#  MIT License
#
#  Copyright (c) 2020 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# createInformaticaPlatformApp
# @Since: 17-JAN-2020
# @Author: Jac. Beekers
# @Version: 20200217.0 - JBE - initial
##

import logging, datetime, supporting
import supporting.errorcodes as err
from cicd.informatica import infaAppChecks
from cicd.informatica import infaSettings as settings
from supporting.generalSettings import logDir
import sys, argparse
from cicd.informatica import infaConstants
from cicd.informatica.application import artifact


now = datetime.datetime.now()
result = err.OK


def parse_the_arguments(argv):
    """Parses the provided arguments and exits on an error.
    Use the option -h on the command line to get an overview of the required and optional arguments.
     """
    parser = argparse.ArgumentParser()
    args = parser.parse_args()

    return args


def main(argv):
    """Creates an Informatica Platform application iar file that can be deployed to a DIS later on
    Usage: createInformaticaPlatformApp.py [-h]
    The module uses environment variables to steer the creation of the archive file
    """
    thisproc = "MAIN"
    mainProc = 'createInformaticaPlatformApp'

    resultlogger = supporting.configurelogger(mainProc)
    logger = logging.getLogger(mainProc)

    args = parse_the_arguments(argv)

    supporting.log(logger, logging.DEBUG, thisproc, 'Started')
    supporting.log(logger, logging.DEBUG, thisproc, 'logDir is >' + logDir + "<.")

    settings.getinfaenvvars()
    settings.outinfaenvvars()

    # Check requirements for artifact generation
    result = infaAppChecks.infa_deploy_checks()
    if result.rc == err.IGNORE.rc:
        # deploylist is not mandatory since 2020-02-09
        supporting.log(logging, result.level, thisproc, 'Apps ignored.')
        result = err.OK
    else:
        if result.rc != err.OK.rc:
            supporting.log(logger, logging.ERROR, thisproc,
                           'Informatica Platform App Checks failed with >' + result.message + "<.")
            supporting.exitscript(resultlogger, result)
        else:
            supporting.log(logger, logging.DEBUG, thisproc, 'Start processing deploy list >' + settings.infa_app_deploylist + "<.")
            result = artifact.processList(infaConstants.CREATE_APP, settings.infa_app_deploylist)
            supporting.log(logger, logging.DEBUG, thisproc, 'Deploy list >' + settings.infa_app_deploylist + "< process returned >" + str(result.rc) +"<.")

    supporting.log(logger, logging.DEBUG, thisproc, 'Completed with return code >' + str(result.rc)
                   + '< and result code >' + result.code + "<.")
    supporting.exitscript(resultlogger, result)


if __name__ == '__main__':
    main(sys.argv)



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

from supporting import log
import logging
import supporting
from cicd.informatica import buildCommand
from cicd.informatica import executeInfacmd
from cicd.informatica import infaSettings
from supporting import errorcodes

class ManageConnection:

    def __init__(self, **keyword_arguments):
        self.logger = logging.getLogger(__name__)
        self.keyword_arguments = keyword_arguments
        self.connection_list =""

    def manage(self):
        RunCommand = buildCommand.build(**self.keyword_arguments)

        log(self.logger, logging.INFO, __name__, "RunCommand is >" + RunCommand + "<.")
        result = executeInfacmd.execute(RunCommand)

        if(result.rc != errorcodes.OK.rc):
            oldResult = result.message
            result = self.keyword_arguments["OnError"]
            result.message = oldResult

        return (result)

    def parseConnectionListOutput(self, outputFile):
        log(self.logger, logging.INFO, __name__, "Parsing outputfile >" + outputFile + "<.")

        with open(outputFile, 'r') as f:
            entireFile = [line.rstrip() for line in f]
        for line in entireFile:
            if 'ID:' in line:
                # found a connection
                connectionName, connectionId = line.split(' - ')
                ignoreThis, connectionId = connectionId.split(': ')
                connectionId = connectionId.rstrip(']')
                log(self.logger, logging.INFO, __name__, "Connection >" + connectionName + "< with id >"+ connectionId +"<.")
                connection_entry = connectionType.strip() + ":" + connectionId.strip() + ":" + connectionName.strip() + "\n"
                self.connection_list += connection_entry
            else:
                # it must be a connection type
                connectionType = line
                log(self.logger, logging.INFO, __name__, "Processing connection type >" + connectionType + "<.")

        return errorcodes.OK

    def writeConnectionList(self, outputFile):
        with open(outputFile, 'w') as f:
            f.write(self.connection_list)

        return errorcodes.OK

    def readConnectionList(self, inputFile):
        with open(inputFile, 'r') as f:
            self.connection_list = [line.rstrip() for line in f]
        return errorcodes.OK

    def getConnectionOptions(self, connection_name, output_file):
        this_proc = 'getConnectionOptions'
        connection = ManageConnection(Tool="ListConnectionOptions",
                                      Domain=infaSettings.sourceDomain,
                                      ConnectionName=connection_name,
                                      OnError=errorcodes.INFACMD_LIST_CONN_OPTIONS_FAILED,
                                      OutputFile=output_file
                                      )
        result = ManageConnection.manage(connection)
        if result.rc != errorcodes.OK.rc:
            with open(output_file, 'r') as f:
                for line in f:
                    result.message += line.rstrip()

        supporting.log(self.logger, logging.DEBUG, this_proc, 'Completed with return code >' + str(result.rc)
                       + '< and result code >' + result.code + "<.")

        return result




#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# Create Oracle Database Artifact
# @Since: 22-MAR-2019
# @Author: Jac. Beekers

import logging, datetime, supporting
import supporting.errorcodes as err
from cicd.database import databaseArtifactChecks as dbchecks
import cicd.database.artifact
import cicd.database.dbSettings as settings
import supporting.generalSettings as generalsettings
import sys, argparse

now = datetime.datetime.now()
result = err.OK
settings.databaseType = 'Oracle'


def parse_the_arguments(argv):
    """Parses the provided arguments and exits on an error.
    Use the option -h on the command line to get an overview of the required and optional arguments.
     """
    parser = argparse.ArgumentParser()
    args = parser.parse_args()

    return args


def main(argv):
    """Creates an Oracle artifact, consisting on collected sql files
    It uses a deploy list that contains schema and init.sql. Check the OracleArtifact docs and examples for more info.
    Module uses environment variables that steer the artifact creation.
    """
    thisproc = "MAIN"
    mainProc = 'CreateOracleArtifact'

    resultlogger = supporting.configurelogger(mainProc)
    logger = logging.getLogger(mainProc)

    args = parse_the_arguments(argv)

    supporting.log(logger, logging.DEBUG, thisproc, 'Started')
    supporting.log(logger, logging.DEBUG, thisproc, 'logDir is >' + generalsettings.logDir + "<.")

    # Check requirements for artifact generation
    generalsettings.getenvvars()
    settings.getdbenvvars()
    settings.outdbenvvars()

    result = dbchecks.databaseartifactchecks()
    if result.rc == err.IGNORE.rc:
        # deploylist is not mandatory since 2020-02-09
        supporting.log(logging, result.level, thisproc, 'Artifact ignored.')
        result = err.OK
    else:
        if result.rc != 0:
            supporting.log(logger, logging.ERROR, thisproc,
                           'Database Artifact Checks failed with >' + result.message + "<.")
            supporting.exitscript(resultlogger, result)
        else:
            result = cicd.database.artifact.processList(settings.dbdeploylist)

    supporting.log(logger, logging.DEBUG, thisproc, 'Completed with return code >' + str(result.rc)
                   + '< and result code >' + result.code + "<.")
    #    supporting.writeresult(resultlogger, result)
    supporting.exitscript(resultlogger, result)


if __name__ == '__main__':
    main(sys.argv)



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

import supporting.errorcodes as err
import supporting.generalConstants as constants
import logging, os, datetime
import supporting

# import repositorytools

now = datetime.datetime.now()
result = err.OK

logger = logging.getLogger(__name__)
workspace = constants.DEFAULT_WORKSPACE


def get_artifact(artifact_name):
    thisproc = "get_artifact"
    global workspace
    # something like this:    artifact = repositorytools.Artifact("group","demoArtifact","1.0.0","classifier","zip")
    # at the moment cicd runs in Azure DevOps and artifacts are stored within the pipeline.

    workspace = get_workspace()
    supporting.log(logger, logging.DEBUG, thisproc, 'workspace is >' + workspace + "<.")

    return workspace + "/" + artifact_name


def get_workspace():
    return os.environ.get(constants.varWorkspace, constants.DEFAULT_WORKSPACE)


def store_artifact(artifact_name):
    thisproc = "store_artifact"
    return err.NOT_IMPLEMENTED



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#


from subprocess import Popen, PIPE
import logging
import supporting.errorcodes as err
from supporting import generalSettings

logger = logging.getLogger(__name__)
from cicd.database import dbSettings
from supporting import log

logger = logging.getLogger(__name__)


class OracleUtilities:

    def __init__(self, db_user, db_password, db_connection, on_sql_error, output_file):
        self.on_sql_error = on_sql_error
        self.output_file = output_file
        self.database_connection = db_connection
        self.connect_string = 'connect ' + db_user + "/" + db_password + "@" + self.database_connection
        if self.on_sql_error == "ABORT":
            self.error_clause_sql = "WHENEVER SQLERROR EXIT SQL.SQLCODE"
            self.error_clause_os = "WHENEVER OSERROR EXIT 9"
            self.spool_clause_on = ""
            self.spool_clause_off = ""
        elif self.on_sql_error == "REPORT":
            self.error_clause_sql = ""
            self.error_clause_os = ""
            self.spool_clause_on = "spool " + self.output_file
            self.spool_clause_off = "spool off"
        else:
            self.error_clause_sql = ""
            self.error_clause_os = ""
            self.spool_clause_on = ""
            self.spool_clause_off = ""

    def run_sqlplus(self, sqlfile):
        thisproc = "run_sqlplus"
        result = err.OK

        try:
            if generalSettings.do_not_run == 'True':
                log(logger, logging.INFO, thisproc,
                    "DO_NOT_RUN is set to True. NOT running script >" + sqlfile + "< on database >"
                    + self.database_connection + "<.")
                script_input = self.error_clause_sql + "\n" + self.error_clause_os + "\n" + self.spool_clause_on + "\n" + self.connect_string + "\n" + "@" + sqlfile + "\n" + self.spool_clause_off + "\n" + "exit 0"
                log(logger, logging.INFO, thisproc, 'input for sqlplus is >' + script_input + '<.')
                return err.OK
            else:
                log(logger, logging.INFO, thisproc,
                    "Running script >" + sqlfile + "< on database >" + self.database_connection + "<.")
                p = Popen([dbSettings.sqlplus_command, "-s", "/NOLOG"], universal_newlines=True, stdin=PIPE,
                          stdout=PIPE,
                          stderr=PIPE)
                stdoutput = p.communicate(input=self.error_clause_sql
                                                + "\n" + self.error_clause_os
                                                + "\n" + self.spool_clause_on
                                                + "\n" + self.connect_string
                                                + "\n" + "@" + sqlfile
                                                + "\n" + self.spool_clause_off
                                                + "\n" + "exit 0")[0]
                log(logger, logging.INFO, thisproc, "SQLPlus output: " + stdoutput)
                if p.returncode == 0:
                    return err.OK
                else:
                    if self.on_sql_error == "ABORT":
                        err.SQLPLUS_ERROR.message = stdoutput
                        return err.SQLPLUS_ERROR
                    else:
                        log(logger, logging.WARNING, thisproc, "Errors occurred but were ignored as on_sql_error is >"
                            + self.on_sql_error + "<. You may want to check >" + self.output_file + "<.")
                        return err.OK

        except FileNotFoundError as e:
            log(logger, logging.ERROR, thisproc, e.strerror + ": " + dbSettings.sqlplus_command)
            return err.SQLFILE_NF



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

#  MIT License
#
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#
#

##
# generalSettings
# @Since: 22-MAR-2019
# @Author: Jac. Beekers
# @Version: 20190412.0 - JBE - Initial
##

import logging, datetime, supporting
import supporting.errorcodes as err
import cicd.informatica.infaArtifactChecks as infachecks
from cicd.informatica import artifact
from cicd.informatica import infaSettings as settings
import supporting.generalSettings as generalsettings
from cicd.informatica import infaConstants
import sys, argparse

now = datetime.datetime.now()
result = err.OK


def parse_the_arguments(argv):
    """Parses the provided arguments and exits on an error.
    Use the option -h on the command line to get an overview of the required and optional arguments.
     """

    parser = argparse.ArgumentParser()
    args = parser.parse_args()

    return args


def main(argv):
    """Generate an Informatica Platform artifact based on environment variables
    """
    thisproc = "MAIN"
    mainProc = 'CreateInformaticaPlatformArtifact'

    resultlogger = supporting.configurelogger(mainProc)
    logger = logging.getLogger(mainProc)

    args = parse_the_arguments(argv)

    supporting.log(logger, logging.DEBUG, thisproc, 'Started')
    supporting.log(logger, logging.DEBUG, thisproc, 'logDir is >' + generalsettings.logDir + "<.")

    settings.getinfaenvvars()
    settings.outinfaenvvars()

    # Check requirements for artifact generation
    # if there is no deploylist, then ignore building the artifact
    result = infachecks.infaartifactchecks()
    if result.rc == err.IGNORE.rc:
        # deploylist is not mandatory since 2020-02-09
        supporting.log(logging, result.level, thisproc, 'Artifact ignored.')
        result = err.OK
    else:
        if result.rc != 0:
            supporting.log(logger, logging.ERROR, thisproc,
                           'Informatica Platform Artifact Checks failed with >' + result.message + "<.")
            supporting.exitscript(resultlogger, result)
        else:
            result = artifact.processList(infaConstants.CREATEARTIFACT, settings.infadeploylist)

    supporting.log(logger, logging.DEBUG, thisproc, 'Completed with return code >' + str(result.rc)
                   + '< and result code >' + result.code + "<.")
    #    supporting.writeresult(resultlogger, result)
    supporting.exitscript(resultlogger, result)


if __name__ == '__main__':
    main(sys.argv)



"""
    Process deploy list for database artifacts
    @Since: 23-MAR-2019
    @Author: Jac. Beekers
    @Version: 20200229.0 - JBE - Infa App added
"""

#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

import supporting.errorcodes as err
import supporting, logging
import supporting.errorcodes as errorcodes
import supporting.deploylist
import supporting.generalSettings as generalSettings
from cicd.informatica.application.artifact import process_deploy_app_entry, process_create_app_entry
from supporting.generalSettings import completePath
from supporting.artifactHandling import get_workspace
import cicd.informatica.infaConstants as infaConstants
import cicd.informatica.infaSettings as infaSettings
from cicd import informatica
from supporting.filehandling import copy_files
import os

logger = logging.getLogger(__name__)
entrynr = 0


def processList(what, deployFile):
    this_function = "processList"
    latest_result = err.OK
    overall_result = err.OK

    supporting.log(logger, logging.DEBUG, this_function, "deployfile is >" + deployFile + "<.")
    result, deploy_items = supporting.deploylist.getWorkitemList(deployFile)
    if result.rc == 0:
        if what == infaConstants.CREATEARTIFACT or what == infaConstants.DEPLOYARTIFACT:
            if what == infaConstants.CREATEARTIFACT:
                supporting.log(logger, logging.DEBUG, this_function,
                           "Copying files in >" + os.path.dirname(deployFile) + "< to artifact.")
                copy_files(os.path.dirname(deployFile), generalSettings.artifactDir)

            # the following also executes if what = deploy artifact
            for deployEntry in deploy_items:
                latest_result = processEntry(what, deployEntry)
                if latest_result.rc != err.OK.rc:
                    overall_result = latest_result
            return overall_result
        else:
            supporting.log(logger, logging.ERROR, this_function, "INTERNAL ERROR: Unexpected 'what' >"+ what +"<.")
    else:
        supporting.log(logger, logging.ERROR, this_function, "Could not get deploylist")
        return errorcodes.FILE_NF


def processEntry(what, deployEntry):
    global entrynr
    thisproc = "processEntry"
    result = err.OK

    entrynr += 1
    supporting.log(logger, logging.DEBUG, thisproc,
                   "Started to work on deploy entry# >" + str(entrynr) + "< being >" + deployEntry + "<.")

    parts = deployEntry.split(':')
    if not len(parts) == 2 and not len(parts) >= 4:
        supporting.log(logger, logging.WARNING, thisproc,
                       "Insufficient entries found. Expected 2 or 4, got >" + str(len(parts)) + "<. Entry IGNORED")
        return err.IGNORE

    type = parts[0]
    object = parts[1]
    if len(parts) >= 4:
        exportcontrol_file = parts[2]
        basename_ecf = exportcontrol_file.split('.')[0]
        export_control = completePath(generalSettings.configDir + "/" + exportcontrol_file, generalSettings.sourceDir)
        supporting.log(logger, logging.DEBUG, thisproc, 'exportcontrolfile is >' + exportcontrol_file
                       + "< and its complete path is >" + export_control + "<. basename is >" + basename_ecf + "<.")

        importcontrol_file = parts[3]
        basename_icf = importcontrol_file.split('.')[0]
        import_control = completePath(infaSettings.target_informatica_dir + "/" + importcontrol_file,
                                      generalSettings.sourceDir)
        supporting.log(logger, logging.DEBUG, thisproc, 'importcontrolfile is >' + importcontrol_file + "<."
                       + "< and its complete path is >" + import_control + "<. basename is >" + basename_icf + "<.")
    else:
        export_control = ""
        import_control = ""

    supporting.log(logger, logging.DEBUG, thisproc, 'Type is >' + type + '< and object is >' + object + '<')
    if what == infaConstants.CREATEARTIFACT:
        result = create_artifact(type, object, export_control, basename_ecf)
    elif what == infaConstants.DEPLOYARTIFACT:
        result = deploy_artifact(type, object, import_control, basename_ecf)
    else:
        result = errorcodes.COMMAND_FAILED

    supporting.log(logger, logging.DEBUG, thisproc,
                   "Completed with rc >" + str(result.rc) + "< and code >" + result.code + "<.")
    return result


def create_artifact(type, object, export_control="default.ecf", export_filename="export"):
    thisproc = 'create_artifact'
    supporting.log(logger, logging.DEBUG, thisproc,
                   "Creating artifact for object >" + object + "< of type >" + type + "<.")

    if type == 'PROJECT':
        result = informatica.export_infadeveloper(
            Domain=infaSettings.sourceDomain,
            Repository=infaSettings.sourceModelRepository,
            Project=object,
            FilePath=generalSettings.artifactDir + "/" + object + "." + export_filename + ".xml",
            OverwriteExportFile=infaSettings.overwriteExportFile,
            ExportRefData=infaSettings.sourceExportRefData
        )
    elif type == 'CONTROLFILE':
        result = informatica.export_infadeveloper(
            Domain=infaSettings.sourceDomain,
            Repository=infaSettings.sourceModelRepository,
            Project=object,
            FilePath=generalSettings.artifactDir + "/" + object + "_" + str(entrynr) + "." + export_filename + ".xml",
            OverwriteExportFile=infaSettings.overwriteExportFile,
            ControlFilePath=export_control
        )
    else:
        result = errorcodes.NOT_IMPLEMENTED

    return result


def deploy_artifact(type, object, import_control, import_filename="export"):
    thisproc = 'deployArtifact'
    supporting.log(logger, logging.DEBUG, thisproc, 'started deploy for object >' + object + '<.')

    #    workspace = get_workspace()
    workspace = infaSettings.target_informatica_dir

    if type == 'PROJECT':
        result = informatica.import_infadeveloper(
            Domain=infaSettings.targetDomain,
            Repository=infaSettings.targetModelRepository,
            Project=object,
            ImportFilePath=workspace + "/" + object + "." + import_filename + ".xml",
            ExportRefData=infaSettings.targetExportRefData
        )
    elif type == 'CONTROLFILE':
        result = informatica.import_infadeveloper(
            Domain=infaSettings.targetDomain,
            Repository=infaSettings.targetModelRepository,
            ImportFilePath=workspace + "/" + object + "_" + str(entrynr) + "." + import_filename + ".xml",
            ControlFilePath=import_control
        )
    else:
        result = errorcodes.NOT_IMPLEMENTED

    return result





#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# Process deploy list for database artifacts
# @Since: 23-MAR-2019
# @Author: Jac. Beekers
# @Version: 20190324.0 - JBE - Initial

import supporting.errorcodes as err
import supporting, logging
import supporting.filehandling as filehandling
import re, os
import cicd.database.dbSettings as settings
import supporting.generalSettings as generalSettings
import supporting.deploylist
from pathlib import Path
from supporting.filehandling import copy_file

logger = logging.getLogger(__name__)
entrynr =0
level =0
previous_schema = 'AUQW&^D*AD&FS'

def processList(deployFile):
    latestError = err.OK
    result, deployItems = supporting.deploylist.getWorkitemList(deployFile)
    if result.rc == 0:
        copy_file(deployFile, generalSettings.artifactDir)
        for deployEntry in supporting.deploylist.deployItems:
            result = processEntry(deployEntry)
            if result.rc != 0:
                latestError = result
    else:
        latestError = result
    return latestError

def processEntry(deployEntry):
    thisproc = "processEntry"
    result = err.OK
    supporting.log(logger, logging.DEBUG, thisproc, "Current directory is >" + os.getcwd() +"<.")
    supporting.log(logger, logging.DEBUG, thisproc, "Started to work on deploy entry >" + deployEntry + "<.")

    schema, sqlfile = deployEntry.split(':', 2)
    supporting.log(logger, logging.DEBUG, thisproc, 'Schema is >' + schema + '< and sqlfile is >' + sqlfile + '<')
    sqlfile = schema + "/" + sqlfile

    sqlfilePath = Path(sqlfile)
    if sqlfilePath.is_file():
        supporting.log(logger, logging.DEBUG, thisproc, 'Found sqlfile >' + sqlfile + "<.")
        sourcesqldir = ""
    else:
        sourcesqldir = settings.sourcesqldir + "/" + settings.databaseType + "/"
        supporting.log(logger, logging.DEBUG, thisproc, 'sqlfile >' + sqlfile + '< not found. Trying >'
                       + sourcesqldir + sqlfile + '<...')
#        sqlfile = sourcesqldir + sqlfile
        sqlfilePath = Path(sourcesqldir + sqlfile)
        if sqlfilePath.is_file():
            supporting.log(logger, logging.DEBUG, thisproc, 'Found sqlfile >' + sourcesqldir + sqlfile + "<.")
        else:
            supporting.log(logger, err.SQLFILE_NF.level, thisproc,
                       "sqlfile checked >" + sourcesqldir + sqlfile + "<. " + err.SQLFILE_NF.message)
            result = err.SQLFILE_NF
            return result

    result = generate_orderedsql(sourcesqldir, schema, sqlfile)

    supporting.log(logger, logging.DEBUG, thisproc,
                   "Completed with rc >" + str(result.rc) + "< and code >" + result.code + "<.")
    return result


def generate_orderedsql(sourcesqldir, schema, input_sqlfile):
    thisproc = "generate_orderedsql"
    global entrynr, previous_schema
    result = err.OK
    supporting.log(logger, logging.DEBUG, thisproc, "Started to work on sql file >" + input_sqlfile + "< in schema >" +schema +"<.")
    supporting.log(logger, logging.DEBUG, thisproc, "settings.targetsqldir is >" + settings.targetsqldir + "<.")

    the_source_sqlfile = input_sqlfile
    if schema == previous_schema:
        entrynr = entrynr + 1
    else:
        entrynr = 1

    prefixReleaseID = settings.sqlprefix + "%02d" % entrynr

    orderedsqlfilename = settings.targetsqldir + "/" + schema + "/" + prefixReleaseID + "_" + schema + "_" + generalSettings.releaseID + ".sql"
    create_directory(settings.targetsqldir + "/" + schema)
    supporting.log(logger, logging.INFO, thisproc,
                   "orderedsqlfilename is >" + orderedsqlfilename + "<. Based on prefixReleaseID >" + prefixReleaseID + ", settings.targetsqldir >"
                   + settings.targetsqldir + "<, schema >" + schema +"< and generalSettings.releaseID >" + generalSettings.releaseID +"<.")

    filehandling.removefile(orderedsqlfilename)
    global level
    level = 0
    result = processlines(sourcesqldir, schema, the_source_sqlfile, orderedsqlfilename)

    supporting.log(logger, logging.DEBUG, thisproc,
                   "Completed with rc >" + str(result.rc) + "< and code >" + result.code + "<.")

    previous_schema = schema
    return result

#TODO: Move to supporting package
def create_directory(directory):
    os.makedirs(directory, exist_ok=True)  # succeeds even if directory exists.


def ignoreline(line):
    if(re.match("^--", line) or re.match("^\n$",line)):
        return True
    return False


def calltosubsql(line):
    thisproc="calltosubsql"
    if(re.match("^@@", line)):
        return True
    return False


def processlines(the_source_sqldir, schema, the_source_sqlfile, orderedsqlfilename):
    result = err.OK
    global level
    level +=1
    thisproc="processlines-" + "%03d" % level
    supporting.log(logger, logging.DEBUG, thisproc, "level is >" + "%03d" % level +"<.")

    try:
        with open(the_source_sqldir + the_source_sqlfile) as thesql:
            for line in thesql:
                if ignoreline(line):
                    continue
                if calltosubsql(line):
                    supporting.log(logger, logging.DEBUG, thisproc, "Found '@@', a call to another script.")
                    write2file(orderedsqlfilename, "-- Start expansion -- " + line)
                    subsql = line[2:-1].split(' ', 1)[0]
                    completepathsql = the_source_sqldir + subsql
                    supporting.log(logger, logging.DEBUG, thisproc, "Sub file name determined as >" + subsql +"<. Complete file path >"
                                   + completepathsql +"<.")
                    processlines(the_source_sqldir, schema, schema +"/" + subsql, orderedsqlfilename)
                    write2file(orderedsqlfilename, "-- End expansion -- " + line)
                else:
                    write2file(orderedsqlfilename, line)

    except IOError:
        supporting.log(logger, logging.ERROR, thisproc, "Could not find file >" + the_source_sqlfile + "<.")
        write2file(orderedsqlfilename,"ERROR: Could not find file >" + the_source_sqlfile +"<.")
        result = err.FILE_NF

    return result

#TODO: Move to supporting package
def write2file(filename, line):
    thisproc="write_sql"
    result = err.OK

    try:
        with open(filename, 'a') as the_result_file:
            if "\n" == line[-1]:
                the_result_file.write(line)
            else:
                the_result_file.write(line +"\n")
    except IOError:
        supporting.log(logger, logging.ERROR, thisproc, "Could not write to file >" + filename + "<.")
        result = err.FILE_NW

    return result



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

import logging, datetime, supporting
from supporting import errorcodes
from cicd.informatica import infaSettings, jobManagement, infaConstants
from supporting import generalSettings
import sys
import argparse

now = datetime.datetime.now()
result = errorcodes.OK


class ExecuteInformaticaScorecard:
    """
        Runs an Informatica Scorecard
    """

    def __init__(self, argv, log_on_console=True, pre_command=None):
        self.arguments = argv
        self.mainProc = 'runScorecard'
        self.pre_command = pre_command
        self.resultlogger = supporting.configurelogger(self.mainProc, log_on_console)
        self.logger = supporting.logger

    def parse_the_arguments(self, arguments):
        """Parses the provided arguments and exits on an error.
        Use the option -h on the command line to get an overview of the required and optional arguments.
        """
        parser = argparse.ArgumentParser(prog='runScorecard')
        parser.add_argument("-s", "--scorecard", required=True, action="store", dest="object_path",
                            help="Scorecard, including path, to run.")
        parser.add_argument("-f", "--osprofile", action="store", dest="os_profile",
                            help="Informatica OSProfile to use.")
        parser.add_argument("-x", "--extra", action="store", dest="as_is_options",
                            help="any options to add. Make sure to use double-quotes!")
        args = parser.parse_args(arguments)

        if args.as_is_options is None:
            args.as_is_options = ""

        return args

    def runit(self, arguments):
        """Runs a Scorecard.
        usage: runScorecard.py [-h] -p OBJECT_PATH
        """
        thisproc = "runit"
        args = self.parse_the_arguments(arguments)

        generalSettings.getenvvars()

        supporting.log(self.logger, logging.DEBUG, thisproc, 'Started')
        supporting.log(self.logger, logging.DEBUG, thisproc, 'logDir is >' + generalSettings.logDir + "<.")

        object_path = args.object_path
        os_profile = args.os_profile
        as_is_options = args.as_is_options

        infaSettings.getinfaenvvars()
        infaSettings.outinfaenvvars()

        scorecard = jobManagement.JobExecution(Tool="RunScorecard",
                                               Domain='$' + infaConstants.varSourceDomain,
                                               DsServiceName='$' + infaConstants.varSourceDIS,
                                               MrsServiceName='$' + infaConstants.varSourceModelRepository,
                                               ObjectPathAndName=object_path,
                                               ObjectType="scorecard",
                                               Wait="true",
                                               OnError=errorcodes.INFACMD_SCORECARD_FAILED,
                                               OperatingSystemProfile=os_profile,
                                               AsIsOptions=as_is_options

                                               )
        result = jobManagement.JobExecution.manage(scorecard)

        supporting.log(self.logger, logging.DEBUG, thisproc, 'Completed with return code >' + str(result.rc)
                       + '< and result code >' + result.code + "<.")
        return result


if __name__ == '__main__':
    infa = ExecuteInformaticaScorecard(sys.argv[1:], log_on_console=True)
    result = infa.runit(infa.arguments)
    supporting.exitscript(infa.resultlogger, result)



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# Informatica constants
# @Since: 10-APR-2019
# @Author: Jac. Beekers
# @License: MIT
# @Version: 20200219.0 - Added Informatica App
import supporting.generalConstants as generalConstants

## Environment variables
# Informatica artifacts and deploys
varDeveloperDeployList = 'DEVELOPER_DEPLOYLIST'
var_developer_app_deploylist = 'DEVELOPER_APP_DEPLOYLIST'

varOverwriteExportFile = 'OVERWRITE_EXPORT_FILE'

varSourceExportRefData = 'SOURCE_EXPORT_REFDATA'
varSourceInfaHome = 'SOURCE_INFA_HOME'
varSourceInfacmdLocation = 'SOURCE_INFACMD_LOCATION'
varSourceDomain = 'SOURCE_INFA_DEFAULT_DOMAIN'
varSourceModelRepository = 'SOURCE_MRS'
varSourceUsername = 'SOURCE_INFA_DEFAULT_DOMAIN_USER'
varSourcePassword = 'SOURCE_INFA_DEFAULT_DOMAIN_PASSWORD'
varSourceSecurityDomain = 'SOURCE_INFA_DEFAULT_SECURITY_DOMAIN'
varSourceDomainInfa = 'SOURCE_INFA_DOMAINS_FILE'
varSourceDIS = 'SOURCE_DIS'
#
varTargetImportRefData = 'TARGET_IMPORT_REFDATA'
varTargetInfaHome = 'TARGET_INFA_HOME'
varTargetInfacmdLocation = 'TARGET_INFACMD_LOCATION'
varTargetDomain = 'TARGET_INFA_DEFAULT_DOMAIN'
varTargetModelRepository = 'TARGET_MRS'
varTargetUsername = 'TARGET_INFA_DEFAULT_DOMAIN_USER'
varTargetPassword = 'TARGET_INFA_DEFAULT_DOMAIN_PASSWORD'
varTargetSecurityDomain = 'TARGET_INFA_DEFAULT_SECURITY_DOMAIN'
varTargetDomainInfa = 'TARGET_INFA_DOMAINS_FILE'
varTargetDIS = 'TARGET_DIS'
varTargetInformaticaDir = 'TARGET_INFORMATICADIR'
varTargetInformaticaAppDir = 'TARGET_INFORMATICA_APP_DIR'

##
# export - import
varExportControlFile = 'EXPORT_CONTROLFILE'
varImportControlFile = 'IMPORT_CONTROLFILE'

# Domains objects
varExportControlFileForDomainObjects = 'EXPORT_DOMAINOBJECTS_CONTROLFILE'
varImportControlFileForDomainObjects = 'IMPORT_DOMAINOBJECTS_CONTROLFILE'
varExportImportConnectionsFile = 'EXPIMP_CONNECTIONS_FILE'
varExportImportConnectionOptionsFile = 'EXPIMP_CONNECTIONOPTIONS_FILE'
vatExportImportUserListFile = 'EXPIMP_USERLIST_FILE'
varExportImportGroupListFile = 'EXPIMP_GROUPLIST_FILE'

# Nexus
varGroupId = "InformaticaPlatform"

##
# Defaults
DEFAULT_DEVELOPER_DEPLOYLIST = 'developer_deploylist.txt'
DEFAULT_DEVELOPER_APP_DEPLOYLIST = 'developer_app_deploylist.txt'

DEFAULT_OVERWRITE_EXPORT_FILE = generalConstants.TRUE
DEFAULT_EXPORT_REFDATA = generalConstants.YES
DEFAULT_IMPORT_REFDATA = generalConstants.YES
DEFAULT_EXPORT_CONTROLFILE = 'ecf_default.xml'
DEFAULT_IMPORT_CONTROLFILE = 'icf_default.xml'
DEFAULT_CONNECTIONSFILE = 'connection_list.txt'
DEFAULT_CONNECTIONOPTIONSFILE = 'connectionoptions_list.txt'
DEFAULT_EXPORT_CONNECTIONSFILE = 'exported_connections.xml'
DEFAULT_IMPORT_CONNECTIONSFILE = 'exported_connections.xml'

##
# Defaults for client or server infaSettings
DEFAULT_INFA_HOME = '/appl/informatica/current'
DEFAULT_DOMAIN = 'DOM_Demo'
DEFAULT_MODEL_REPOSITORY = 'MRS_Demo'
DEFAULT_USERNAME = 'notReally'
DEFAULT_PASSWORD = 'W3Akrdu+PECXwqC/W21nXQ=='
DEFAULT_SECURITYDOMAIN = 'Native'
DEFAULT_DATAINTEGRATION_SERVICE = 'DIS_Demo01'
DEFAULT_TARGET_INFORMATICADIR = '.'
DEFAULT_TARGET_INFORMATICA_APP_DIR = '.'

##
# Generic stuff
NOT_PROVIDED = "NotProvided"
NOT_FOUND = "NotFound"

##
# PowerCenter
varPowerCenterDeployList = 'POWERCENTER_DEPLOOYLIST'
DEFAULT_POWERCENTER_DEPLOYLIST = 'powercenter_deploylist.txt'

##
#
CREATEARTIFACT = 'CreateArtifact'
DEPLOYARTIFACT = 'DeployArtifact'
CREATE_APP = 'CreateApp'
DEPLOY_APP = 'DeployApp'

##
# Informatica Connections
varConnectionPassword = 'INFA_DEFAULT_CONNECTION_PASSWORD'

##
# Lists
AvailableArguments = {
    "Domain": "-DomainName",
    "User": "-UserName",
    "Service": "-sn",
    "SecurityDomain": "-SecurityDomain",
    "Repository": "-RepositoryService",
    "TargetFolder": "-tf",
    "ConflictResolution": "-cr",
    "FilePath": "-ExportFilePath",
    "ImportFilePath": "-ImportFilePath",
    "Path": "-p",
    "SourceProject": "-sp",
    "TargetProject": "-tp",
    "Project": "-ProjectName",
    "SkipCRC": "-sc",
    "ControlFilePath": "-ControlFilePath",
    "OverwriteExportFile": "-OverwriteExportFile",
    "ByObjectPathName": "-bopn",
    "ByUser": "-bu",
    "ObjectPathName": "-opn",
    "OtherOptions": "-OtherOptions",

}

AvailableTools = {
    "Import": ("oie", "ImportObjects"),
    "Export": ("oie", "ExportObjects"),
    "CreateIAR": ("tools", "deployApplication"),
    "DeployIAR": ("dis", "deployApplication"),
    "RedeployIAR": ("dis", "updateApplication"),
    "CreateUser": ("isp", "CreateUser"),
    "DisableUser": ("isp", "DisableUser"),
    "DeleteUser": ("isp", "RemoveUser"),
    "CreateGroup": ("isp", "CreateGroup"),
    "DeleteGroup": ("isp", "RemoveGroup"),
    "ExportUsersAndGroups": ("isp", "exportUsersAndGroups"),
    "ImportUsersAndGroups": ("isp", "importUsersAndGroups"),
    "CreateProject": ("mrs", "CreateProject"),
    "DeleteProject": ("mrs", "DeleteProject"),
    "CreateFolder": ("mrs", "CreateFolder"),
    "DeleteFolder": ("mrs", "DeleteFolder"),
    "ListCheckOutObjects": ("mrs", "listCheckedOutObjects"),
    "CheckIn": ("mrs", "checkInObject"),
    "RunProfile": ("ps", "Execute"),
    "RunScorecard": ("ps", "Execute"),
    "RunMapping": ("ms", "runMapping"),
    "RunWorkflow": ("wfs", "startWorkflow"),
    "ListConnections": ("isp", "listConnections"),
    "ListConnectionOptions": ("isp", "listConnectionOptions"),
    "CreateConnection": ("isp", "createConnection"),
    "DeleteConnection": ("isp", "removeConnection"),
    "UpdateConnection": ("isp", "updateConnection"),
    "AddConnectionPermissions": ("isp", "addConnectionPermissions"),
    "ListConnectionPermissions": ("isp", "listConnectionPermissions"),
    "RemoveConnectionPermissions": ("isp", "removeConnectionPermissions"),
    "SetConnectionPermissions": ("isp", "setConnectionPermissions"),
    "ExportConnections": ("isp", "exportDomainObjects"),
    "ImportConnections": ("isp", "importDomainObjects"),
    "ListApplications": ("dis", "listApplications"),
    "StopApp": ("dis", "stopApplication"),
    "StartApp": ("dis", "startApplication"),

}

##
# Informatica Optimization Levels
optimization_level = {
    "Auto": -1,
    "None": 0,
    "Minimal": 1,
    "Normal": 2,
    "Full": 3
}

##
# Informatica error codes captured
informatica_app_already_exists = 'DSCMN_10020'
